from collections.abc import Callable
from contextlib import AbstractContextManager
from types import TracebackType
from typing import Any, Generic, ParamSpec, TypeVar, overload

from typing_extensions import Never, override

__all__ = ("Bunch", "FallbackContext", "getitem_property", "mro_lookup")

class Bunch:
    def __init__(self, **kwargs: Any) -> None: ...

def mro_lookup(
    cls: type[Any],
    attr: str,
    stop: set[type[Any]] | None = None,
    monkey_patched: list[Any] | None = None,
) -> type[Any] | None: ...

_P = ParamSpec("_P")
_T_co = TypeVar("_T_co", bound=object, covariant=True)

class FallbackContext(
    AbstractContextManager[_T_co],
    Generic[_T_co, _P],
):
    provided: _T_co | None
    fallback: Callable[_P, AbstractContextManager[_T_co]] | None
    fb_args: tuple[Any, ...]
    fb_kwargs: dict[str, Any]

    def __init__(
        self,
        provided: _T_co | None,
        fallback: Callable[_P, AbstractContextManager[_T_co]] | None,
        *fb_args: Any,
        **fb_kwargs: Any,
    ) -> None: ...
    @overload
    def __new__(
        cls,
        provided: None,
        fallback: None,
    ) -> Never: ...
    @overload
    def __new__(
        cls,
        provided: _T_co,
        fallback: None,
    ) -> FallbackContext[_T_co, _P]: ...
    @overload
    def __new__(
        cls,
        provided: None,
        fallback: Callable[_P, AbstractContextManager[_T_co]],
        *fb_args: _P.args,
        **fb_kwargs: _P.kwargs,
    ) -> FallbackContext[_T_co, _P]: ...
    @override
    def __enter__(self) -> _T_co: ...
    @override
    def __exit__(
        self,
        typ: type[BaseException] | None,
        exc: BaseException | None,
        tb: TracebackType | None,
    ) -> bool | None: ...

class getitem_property:
    path: list[str] | None
    key: str
    __doc__: str | None

    def __init__(self, keypath: str, doc: str | None = None) -> None: ...
    def _path(self, obj: Any) -> dict[str, Any]: ...
    def __get__(self, obj: Any, type: Any | None = None) -> Any: ...
    def __set__(self, obj: Any, value: Any) -> None: ...



from typing import Any

__all__ = [
    "_main",
    "freeze_support",
    "get_command_line",
    "get_executable",
    "get_preparation_data",
    "import_main_path",
    "set_executable",
]

def get_command_line(**kwds: Any) -> list[str]: ...
def get_executable() -> str: ...
def get_preparation_data(name: str) -> dict[str, Any]: ...
def import_main_path(main_path: str) -> None: ...
def set_executable(exe: str) -> None: ...
def freeze_support() -> None: ...
def _main(fd: int) -> None: ...



from collections.abc import Callable
from typing import Any

from billiard.pool import Pool as BilliardPool
from celery.concurrency.asynpool import AsynPool
from celery.concurrency.base import BasePool
from typing_extensions import override

__all__ = ("TaskPool", "process_destructor", "process_initializer")

def process_initializer(app: Any, hostname: str) -> None: ...
def process_destructor(pid: int, exitcode: int) -> None: ...

class TaskPool(BasePool):
    Pool: type[AsynPool]
    BlockingPool: Callable[..., BilliardPool]
    uses_semaphore: bool
    write_stats: Callable[[], str | None] | None

    @override
    def on_start(self) -> None: ...
    @override
    def restart(self) -> None: ...
    @override
    def on_stop(self) -> None: ...
    @override
    def on_terminate(self) -> None: ...
    @override
    def on_close(self) -> None: ...



import datetime
from collections.abc import Callable, Sequence
from typing import Any, Literal, TypedDict

__all__ = ("Request",)

from celery.app.base import Celery
from celery.app.task import Task
from celery.canvas import chord as chord_type
from celery.concurrency.base import BasePool
from celery.result import AsyncResult

class _DeliveryInfo(TypedDict, total=False):
    exchange: str | None
    routing_key: str | None
    priority: int | None
    redelivered: bool | None

class _InfoDict(TypedDict):
    id: str
    name: str
    args: Any
    kwargs: Any
    type: str
    hostname: str
    time_start: float
    acknowledged: bool
    delivery_info: _DeliveryInfo

class Request:
    acknowledged: bool
    time_start: float | None
    worker_pid: str | None
    time_limits: tuple[float | None, float | None]
    id: str
    def __init__(
        self,
        message: Any,
        on_ack: Callable[..., Any] = ...,
        hostname: str | None = ...,
        eventer: Any | None = ...,
        app: Celery | None = ...,
        connection_errors: Sequence[Any] | None = ...,
        request_dict: object = ...,
        task: Task[Any, Any] | None = ...,
        on_reject: Callable[..., Any] = ...,
        body: Any | None = ...,
        headers: dict[str, str] | None = ...,
        decoded: bool = ...,
        utc: bool = ...,
        maybe_make_aware: Callable[[datetime.datetime], datetime.datetime] = ...,
        maybe_iso8601: Callable[[datetime.datetime | str], datetime.datetime] = ...,
        **opts: Any,
    ) -> None: ...
    @property
    def delivery_info(self) -> _DeliveryInfo: ...
    @property
    def message(self) -> Any: ...
    @property
    def request_dict(self) -> dict[str, Any]: ...
    @property
    def body(self) -> Any: ...
    @property
    def app(self) -> Celery | None: ...
    @property
    def utc(self) -> bool: ...
    @property
    def content_type(self) -> str: ...
    @property
    def content_encoding(self) -> str: ...
    @property
    def type(self) -> str: ...
    @property
    def root_id(self) -> str | None: ...
    @property
    def parent_id(self) -> str | None: ...
    @property
    def argsrepr(self) -> str: ...
    @property
    def args(self) -> Any: ...
    @property
    def kwargs(self) -> Any: ...
    @property
    def kwargsrepr(self) -> str: ...
    @property
    def on_ack(self) -> Callable[..., Any]: ...
    @property
    def on_reject(self) -> Callable[..., Any]: ...
    @on_reject.setter
    def on_reject(self, value: Callable[..., Any]) -> None: ...
    @property
    def hostname(self) -> str: ...
    @property
    def eventer(self) -> Any: ...
    @eventer.setter
    def eventer(self, eventer: Any) -> None: ...
    @property
    def connection_errors(self) -> Sequence[Any]: ...
    @property
    def task(self) -> Task[Any, Any]: ...
    @property
    def eta(self) -> datetime.datetime | None: ...
    @property
    def expires(self) -> datetime.datetime: ...
    @expires.setter
    def expires(self, value: datetime.datetime) -> None: ...
    @property
    def tzlocal(self) -> Any: ...
    @property
    def store_errors(self) -> bool: ...
    @property
    def task_id(self) -> str: ...
    @task_id.setter
    def task_id(self, value: str) -> None: ...
    @property
    def task_name(self) -> str: ...
    @task_name.setter
    def task_name(self, value: str) -> None: ...
    @property
    def reply_to(self) -> Any: ...
    @property
    def correlation_id(self) -> Any: ...
    @property
    def name(self) -> str: ...
    @property
    def ignore_result(self) -> bool: ...
    @property
    def stamped_headers(self) -> list[str] | None: ...
    @property
    def stamps(self) -> dict[str, Any]: ...
    @property
    def groups(self) -> list[str] | None: ...
    @property
    def replaced_task_nesting(self) -> int: ...
    def cancel(
        self, pool: BasePool, signal: int | None = ..., emit_retry: bool = True
    ) -> None: ...
    def execute_using_pool(self, pool: BasePool, **kwargs: Any) -> AsyncResult[Any]: ...
    def execute(self, loglevel: int | None = ..., logfile: str | None = ...) -> Any: ...
    def maybe_expire(self) -> None | Literal[True]: ...
    def terminate(self, pool: BasePool, signal: int | None = ...) -> None: ...
    def revoked(self) -> bool: ...
    def send_event(self, type: str, **fields: Any) -> None: ...
    def on_accepted(self, pid: str, time_accepted: float) -> None: ...
    def on_timeout(self, soft: bool, timeout: float) -> None: ...
    def on_success(
        self, failed__retval__runtime: tuple[Any, Any, Any], **kwargs: Any
    ) -> None: ...
    def on_retry(self, exc_info: Any) -> None: ...
    def on_failure(
        self, exc_info: Any, send_failed_event: bool = ..., return_ok: bool = ...
    ) -> None: ...
    def acknowledge(self) -> None: ...
    def reject(self, requeue: bool = ...) -> None: ...
    def info(self, safe: bool = ...) -> _InfoDict: ...
    def humaninfo(self) -> str: ...
    @property
    def chord(self) -> chord_type | None: ...
    @property
    def errbacks(self) -> Any | None: ...
    @property
    def group(self) -> Any | None: ...
    @property
    def group_index(self) -> Any | None: ...



from typing import IO, Any

from celery.apps.multi import (
    Cluster as _Cluster,
)
from celery.apps.multi import (
    MultiParser as _MultiParser,
)
from celery.apps.multi import (
    NamespacedOptionParser as _NamespacedOptionParser,
)
from celery.bin.base import CeleryCommand
from celery.utils.term import colored

__all__ = ("MultiTool",)

class TermLogger:
    retcode: int
    splash_text: str
    splash_context: dict[str, Any]
    stdout: IO[str]
    stderr: IO[str]
    nosplash: bool
    quiet: bool
    verbose: bool
    no_color: bool

    @property
    def colored(self) -> colored: ...  # ty: ignore[invalid-type-form]
    def setup_terminal(
        self,
        stdout: IO[str] | None,
        stderr: IO[str] | None,
        nosplash: bool = False,
        quiet: bool = False,
        verbose: bool = False,
        no_color: bool = False,
        **kwargs: Any,
    ) -> None: ...
    def ok(self, m: str, newline: bool = True, file: IO[str] | None = None) -> int: ...
    def say(
        self, m: str, newline: bool = True, file: IO[str] | None = None
    ) -> None: ...
    def carp(
        self, m: str, newline: bool = True, file: IO[str] | None = None
    ) -> int: ...
    def error(self, msg: str | None = None) -> int: ...
    def info(self, msg: str, newline: bool = True) -> None: ...
    def note(self, msg: str, newline: bool = True) -> None: ...
    def usage(self) -> None: ...
    def splash(self) -> None: ...

class MultiTool(TermLogger):
    MultiParser: type[_MultiParser]
    OptionParser: type[_NamespacedOptionParser]
    reserved_options: list[tuple[str, str]]
    env: dict[str, str] | None
    cmd: str | None
    fh: IO[str]
    prog_name: str
    commands: dict[str, Any]

    @property
    def OK(self) -> int: ...
    @property
    def DOWN(self) -> int: ...
    @property
    def FAILED(self) -> int: ...
    def __init__(
        self,
        env: dict[str, str] | None = None,
        cmd: str | None = None,
        fh: IO[str] | None = None,
        stdout: IO[str] | None = None,
        stderr: IO[str] | None = None,
        **kwargs: Any,
    ) -> None: ...
    def Cluster(self, nodes: list[Any], cmd: str | None = None) -> _Cluster: ...
    def execute_from_commandline(
        self, argv: list[str], cmd: str | None = None
    ) -> int: ...
    def validate_arguments(self, argv: list[str]) -> bool: ...
    def call_command(self, command: str, argv: list[str]) -> int: ...
    def cluster_from_argv(
        self, argv: list[str], cmd: str | None = None
    ) -> _Cluster: ...
    def start(self, cluster: _Cluster) -> int: ...
    def stop(self, cluster: _Cluster, sig: int, **kwargs: Any) -> int: ...
    def stopwait(self, cluster: _Cluster, sig: int, **kwargs: Any) -> int: ...
    stop_verify: Any  # alias for stopwait
    def restart(self, cluster: _Cluster, sig: int, **kwargs: Any) -> int: ...
    def names(self, cluster: _Cluster) -> None: ...
    def get(self, wanted: str, *argv: str) -> int: ...
    def show(self, cluster: _Cluster) -> None: ...
    def expand(self, template: str, *argv: str) -> None: ...
    def help(self, *argv: str) -> None: ...
    def kill(self, cluster: _Cluster) -> None: ...
    def on_node_start(self, node: Any) -> None: ...
    def on_node_restart(self, node: Any) -> None: ...
    def on_node_shutdown_ok(self, node: Any) -> None: ...
    def on_node_status(self, node: Any, retval: int) -> None: ...
    def on_node_signal(self, node: Any, sig: int) -> None: ...
    def on_node_signal_dead(self, node: Any) -> None: ...
    def on_node_down(self, node: Any) -> None: ...
    def on_child_spawn(self, node: Any, argstr: str, env: dict[str, str]) -> None: ...
    def on_child_signalled(self, node: Any, signum: int) -> None: ...
    def on_child_failure(self, node: Any, retcode: int) -> None: ...
    def on_send_signal(self, node: Any, sig: int) -> None: ...
    def on_still_waiting_for(self, nodes: list[Any]) -> None: ...
    def on_still_waiting_progress(self, nodes: list[Any]) -> None: ...
    def on_still_waiting_end(self) -> None: ...
    def on_stopping_preamble(self, nodes: list[Any]) -> None: ...

multi: CeleryCommand



from typing import Any, ClassVar

from celery.bootsteps import Step
from typing_extensions import override

__all__ = ("DelayedDelivery",)

class DelayedDelivery(Step):
    name: ClassVar[str]  # pyright: ignore[reportIncompatibleVariableOverride]

    @override
    def include_if(self, c: Any) -> bool: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]
    def start(self, c: Any) -> None: ...



from typing import Any

from celery.backends.base import KeyValueStoreBackend

__all__ = ("DynamoDBBackend",)

class DynamoDBBackend(KeyValueStoreBackend):
    aws_region: str | None
    table_name: str
    endpoint_url: str | None
    read_capacity_units: int
    write_capacity_units: int
    time_to_live_seconds: int | None

    @property
    def client(self) -> Any: ...
    def __init__(
        self,
        url: str | None = None,
        table_name: str | None = None,
        *args: Any,
        **kwargs: Any,
    ) -> None: ...



from collections.abc import Callable, Iterable, Iterator, Sequence
from typing import Any, Generic, TypeVar, overload

__all__ = ("FairCycle", "priority_cycle", "round_robin_cycle", "sorted_cycle")

_T = TypeVar("_T")

def cycle_by_name(
    name: str,
) -> type[priority_cycle[Any] | round_robin_cycle[Any] | sorted_cycle[Any]]: ...

class round_robin_cycle(Generic[_T]):
    items: list[_T] | None
    @overload
    def __init__(self, it: Iterable[_T]) -> None: ...
    @overload
    def __init__(self: round_robin_cycle[Any], it: None = ...) -> None: ...
    def update(self, it: Iterable[_T]) -> None: ...
    def consume(self, n: int) -> Iterator[_T]: ...
    def rotate(self, last_used: _T) -> None: ...

class priority_cycle(round_robin_cycle[_T]): ...
class sorted_cycle(round_robin_cycle[_T]): ...

class FairCycle:
    fun: Callable[..., Any]
    resources: Sequence[Any]
    predicate: type[BaseException]
    pos: int

    def __init__(
        self,
        fun: Callable[..., Any],
        resources: Sequence[Any],
        predicate: type[BaseException] = ...,
    ) -> None: ...
    def _next(self) -> Any: ...
    def get(self, callback: Callable[..., _T] | None, **kwargs: Any) -> _T | None: ...
    def close(self) -> None: ...



from collections.abc import Callable, Mapping
from logging import Logger
from typing import Any, NamedTuple

ssl_available: bool
safequote: Callable[..., str]
logger: Logger

class urlparts(NamedTuple):
    scheme: str
    hostname: str | None
    port: int | None
    username: str | None
    password: str | None
    path: str | None
    query: Mapping[str, Any]

def _parse_url(url: str) -> urlparts: ...
def parse_url(url: str) -> dict[str, Any]: ...
def url_to_parts(url: str) -> urlparts: ...
def as_url(
    scheme: str,
    host: str | None = ...,
    port: int | None = ...,
    user: str | None = ...,
    password: str | None = ...,
    path: str | None = ...,
    query: dict[str, Any] | None = ...,
    sanitize: bool = ...,
    mask: str = ...,
) -> str: ...
def sanitize_url(url: str, mask: str = ...) -> str: ...
def maybe_sanitize_url(url: str, mask: str = ...) -> str: ...
def parse_ssl_cert_reqs(query_value: str | None) -> int | None: ...



from typing import Any

__all__ = ("SessionManager",)

class SessionManager:
    def __init__(self) -> None: ...
    def get_engine(self, dburi: str, **kwargs: Any) -> Any: ...
    def create_session(
        self, dburi: str, short_lived_sessions: bool = False, **kwargs: Any
    ) -> Any: ...
    def prepare_models(self, engine: Any) -> None: ...
    def session_factory(self, dburi: str, **kwargs: Any) -> Any: ...



import numbers
from datetime import datetime
from typing import Any

from billiard.exceptions import (
    SoftTimeLimitExceeded,
    Terminated,
    TimeLimitExceeded,
    WorkerLostError,
)
from celery.canvas import Signature
from click.exceptions import ClickException
from kombu.exceptions import OperationalError

__all__ = (
    "AlreadyRegistered",
    "AlwaysEagerIgnored",
    "BackendError",
    "BackendGetMetaError",
    "BackendStoreError",
    "CDeprecationWarning",
    "CPendingDeprecationWarning",
    "CeleryCommandException",
    "CeleryError",
    "CeleryWarning",
    "ChordError",
    "DuplicateNodenameWarning",
    "FixupWarning",
    "Ignore",
    "ImproperlyConfigured",
    "IncompleteStream",
    "InvalidTaskError",
    "MaxRetriesExceededError",
    "NotConfigured",
    "NotRegistered",
    "OperationalError",
    "QueueNotFound",
    "Reject",
    "Retry",
    "SecurityError",
    "SecurityWarning",
    "SoftTimeLimitExceeded",
    "TaskError",
    "TaskPredicate",
    "TaskRevokedError",
    "Terminated",
    "TimeLimitExceeded",
    "TimeoutError",
    "WorkerLostError",
    "WorkerShutdown",
    "WorkerTerminate",
    "reraise",
)

class CeleryWarning(UserWarning): ...
class AlwaysEagerIgnored(CeleryWarning): ...
class DuplicateNodenameWarning(CeleryWarning): ...
class FixupWarning(CeleryWarning): ...
class NotConfigured(CeleryWarning): ...
class CeleryError(Exception): ...
class TaskPredicate(CeleryError): ...

class Retry(TaskPredicate):
    message: str | None
    exc: Exception | None
    when: numbers.Real | datetime | None
    def __init__(
        self,
        message: str | None = ...,
        exc: Exception | None = ...,
        when: numbers.Real | datetime | None = ...,
        is_eager: bool = ...,
        sig: Signature[Any] | None = ...,
        **kwargs: object,
    ) -> None: ...
    def humanize(self) -> str: ...

RetryTaskError = Retry

class Ignore(TaskPredicate): ...

class Reject(TaskPredicate):
    def __init__(
        self, reason: Exception | str | None = ..., requeue: bool | None = ...
    ) -> None: ...

class ImproperlyConfigured(CeleryError): ...
class SecurityError(CeleryError): ...
class TaskError(CeleryError): ...
class QueueNotFound(KeyError, TaskError): ...
class IncompleteStream(TaskError): ...
class NotRegistered(KeyError, TaskError): ...
class AlreadyRegistered(TaskError): ...
class TimeoutError(TaskError): ...

class MaxRetriesExceededError(TaskError):
    task_args: list[Any]
    task_kwargs: dict[str, Any]
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...

class TaskRevokedError(TaskError): ...
class InvalidTaskError(TaskError): ...
class ChordError(TaskError): ...
class CPendingDeprecationWarning(PendingDeprecationWarning): ...
class CDeprecationWarning(DeprecationWarning): ...
class WorkerTerminate(SystemExit): ...

SystemTerminate = WorkerTerminate

class WorkerShutdown(SystemExit): ...
class BackendError(Exception): ...
class BackendGetMetaError(BackendError): ...
class BackendStoreError(BackendError): ...
class SecurityWarning(CeleryWarning): ...

def reraise(tp: type[BaseException], value: BaseException, tb: Any = None) -> None: ...

class CeleryCommandException(ClickException):
    def __init__(self, message: str, exit_code: int) -> None: ...



from typing import Any

from azure.servicebus import ServiceBusClient, ServiceBusReceiver, ServiceBusSender
from azure.servicebus.management import ServiceBusAdministrationClient
from kombu.transport.virtual import Channel as VirtualChannel
from kombu.transport.virtual import Transport as VirtualTransport
from kombu.utils.objects import cached_property
from typing_extensions import override

PUNCTUATIONS_TO_REPLACE: set[str]
CHARS_REPLACE_TABLE: dict[int, int]

class SendReceive:
    receiver: ServiceBusReceiver | None
    sender: ServiceBusSender | None

    def __init__(
        self,
        receiver: ServiceBusReceiver | None = ...,
        sender: ServiceBusSender | None = ...,
    ) -> None: ...
    def close(self) -> None: ...

class Channel(VirtualChannel):
    default_wait_time_seconds: int
    default_peek_lock_seconds: int
    default_uamqp_keep_alive_interval: int
    default_retry_total: int
    default_retry_backoff_factor: float
    default_retry_backoff_max: int
    domain_format: str
    _queue_cache: dict[str, SendReceive]
    _noack_queues: set[str]

    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    @override
    def basic_consume(
        self, queue: str, no_ack: bool, *args: Any, **kwargs: Any
    ) -> str: ...
    @override
    def basic_cancel(self, consumer_tag: str) -> None: ...
    def entity_name(self, name: str, table: dict[int, int] | None = ...) -> str: ...
    @override
    def basic_ack(self, delivery_tag: str, multiple: bool = ...) -> None: ...
    @override
    def close(self) -> None: ...
    @cached_property
    def queue_service(self) -> ServiceBusClient: ...
    @cached_property
    def queue_mgmt_service(self) -> ServiceBusAdministrationClient: ...
    @property
    def conninfo(self) -> Any: ...
    @property
    def transport_options(self) -> dict[str, Any]: ...
    @cached_property
    def queue_name_prefix(self) -> str: ...
    @cached_property
    def wait_time_seconds(self) -> int: ...
    @cached_property
    def peek_lock_seconds(self) -> int: ...
    @cached_property
    def uamqp_keep_alive_interval(self) -> int: ...
    @cached_property
    def retry_total(self) -> int: ...
    @cached_property
    def retry_backoff_factor(self) -> float: ...
    @cached_property
    def retry_backoff_max(self) -> int: ...

_Channel = Channel

class Transport(VirtualTransport):
    Channel: type[_Channel]  # pyright: ignore[reportIncompatibleVariableOverride]

    polling_interval: int  # pyright: ignore[reportIncompatibleVariableOverride]
    default_port: None  # pyright: ignore[reportIncompatibleVariableOverride]
    can_parse_url: bool

    @staticmethod
    def parse_uri(uri: str) -> tuple[str, Any]: ...
    @classmethod
    @override
    def as_uri(cls, uri: str, include_password: bool = ..., mask: str = ...) -> str: ...  # pyright: ignore[reportIncompatibleMethodOverride]



from collections import defaultdict
from queue import Queue
from typing import Any

from kombu.transport.virtual import Channel as VirtualChannel
from kombu.transport.virtual import Transport as VirtualTransport
from kombu.transport.virtual.base import BrokerState
from typing_extensions import override

class Channel(VirtualChannel):
    events: defaultdict[str, set[Any]]
    queues: dict[str, Queue[Any]]
    do_restore: bool
    supports_fanout: bool

    def __init__(self, connection: Any, **kwargs: Any) -> None: ...
    @override
    def close(self) -> None: ...
    def _queue_for(self, queue: str) -> Queue[Any]: ...
    def _put_fanout(
        self,
        exchange: str,
        message: dict[str, Any],
        routing_key: str | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def _queue_bind(self, *args: Any) -> None: ...

_Channel = Channel

class Transport(VirtualTransport):
    Channel: type[_Channel]  # pyright: ignore[reportIncompatibleVariableOverride]
    default_port: int | None
    driver_type: str
    driver_name: str
    global_state: BrokerState



from collections.abc import Callable, Iterator
from contextlib import contextmanager
from typing import Any, TextIO

__all__ = (
    "blockdetection",
    "cry",
    "humanbytes",
    "mem_rss",
    "memdump",
    "ps",
    "sample",
    "sample_mem",
)

UNITS: tuple[tuple[float, str], ...]

def blockdetection(timeout: float) -> Any: ...
def cry(
    out: TextIO | Callable[[str], None] | None = None,
    sepchr: str = "=",
    seplen: int = 49,
) -> str: ...
def hfloat(f: float, p: int = 5) -> str: ...
def humanbytes(s: float) -> str: ...
def mem_rss() -> float: ...
def memdump(samples: int = 10, file: TextIO | None = None) -> None: ...
def ps() -> Any: ...
def sample(x: int, n: int, k: int = 0) -> list[Any]: ...
@contextmanager
def sample_mem() -> Iterator[Callable[[], str]]: ...



from collections.abc import Callable, Generator, Iterable
from contextlib import contextmanager
from typing import Any
from uuid import UUID

from kombu.connection import Connection
from kombu.entity import Exchange, Queue
from kombu.messaging import Consumer, Producer
from kombu.transport.base import StdChannel

__all__ = (
    "Broadcast",
    "collect_replies",
    "drain_consumer",
    "eventloop",
    "insured",
    "itermessages",
    "maybe_declare",
    "send_reply",
    "uuid",
)

class Broadcast(Queue):
    def __init__(
        self,
        name: str | None = ...,
        queue: str | None = ...,
        unique: bool = ...,
        auto_delete: bool = ...,
        exchange: Exchange | None = ...,
        alias: str | None = ...,
        **kwargs: Any,
    ) -> None: ...

class QoS:
    callback: Callable[..., None]
    prev: int | None
    value: int
    max_prefetch: int | None
    _mutex: Any

    def __init__(
        self,
        callback: Callable[..., None],
        initial_value: int,
        max_prefetch: int | None = ...,
    ) -> None: ...
    def update(self) -> int: ...
    def decrement_eventually(self, n: int = ...) -> int: ...
    def increment_eventually(self, n: int = ...) -> int: ...
    def set(self, pcount: int) -> int: ...

@contextmanager
def _ignore_errors(conn: Connection) -> Generator[None, None, None]: ...
def _ensure_channel_is_bound(entity: Any, channel: StdChannel | None) -> StdChannel: ...
def _maybe_declare(entity: Any, channel: StdChannel) -> bool: ...
def _imaybe_declare(
    entity: Any, channel: StdChannel, **retry_policy: Any
) -> Generator[Any, None, bool]: ...
def _ensure_errback(exc: BaseException, interval: float) -> None: ...
def maybe_declare(
    entity: Any,
    channel: StdChannel | None = ...,
    retry: bool = ...,
    **retry_policy: Any,
) -> bool: ...
def uuid(_uuid: Callable[[], UUID] = ...) -> str: ...
def itermessages(
    conn: Connection,
    channel: StdChannel,
    queue: Queue,
    limit: int = ...,
    timeout: float | None = ...,
    callbacks: Iterable[Callable[..., Any]] | None = ...,
    **kwargs: Any,
) -> Generator[tuple[Any, Any], None, None]: ...
def send_reply(
    exchange: Exchange,
    req: Any,
    msg: Any,
    producer: Producer | None = ...,
    retry: bool = ...,
    retry_policy: dict[str, Any] | None = ...,
    **props: Any,
) -> None: ...
def collect_replies(
    conn: Connection,
    channel: StdChannel,
    queue: Queue,
    *args: Any,
    **kwargs: Any,
) -> Generator[Any, None, None]: ...
def insured(
    pool: Any,
    fun: Callable[..., Any],
    args: tuple[Any, ...],
    kwargs: dict[str, Any],
    errback: Callable[..., Any] | None = ...,
    on_revive: Callable[..., Any] | None = ...,
    **opts: Any,
) -> Any: ...
def drain_consumer(
    consumer: Consumer,
    limit: int = ...,
    timeout: float | None = ...,
    callbacks: Iterable[Callable[..., Any]] | None = ...,
) -> Generator[tuple[Any, Any], None, None]: ...
def eventloop(
    conn: Connection,
    limit: int | None = ...,
    timeout: float | None = ...,
    ignore_timeouts: bool = ...,
) -> Generator[Any, None, None]: ...
def ignore_errors(
    conn: Connection, fun: Callable[..., Any] | None = ..., *args: Any, **kwargs: Any
) -> Any: ...
def revive_connection(
    connection: Connection,
    channel: StdChannel,
    on_revive: Callable[..., Any] | None = ...,
) -> StdChannel: ...
def declaration_cached(entity: Any, channel: StdChannel) -> bool: ...
def oid_from(instance: Any, threads: bool = ...) -> str: ...
def get_node_id() -> str: ...
def generate_oid(
    node_id: str, process_id: int, thread_id: int, instance: Any
) -> str: ...



from datetime import timedelta
from uuid import UUID

from django.db import models
from django.db.models.query import QuerySet
from django_celery_results.models import GroupResult, TaskResult

class TaskResultManager(models.Manager[TaskResult]):
    def get_task(self, task_id: str) -> TaskResult: ...
    def store_result(
        self,
        content_type: str,
        content_encoding: str,
        task_id: str | UUID,
        result: str,
        status: str,
        traceback: str | None = ...,
        meta: str | None = ...,
        task_name: str | None = ...,
        task_args: str | None = ...,
        task_kwargs: str | None = ...,
        using: str | None = ...,
    ) -> TaskResult: ...
    def get_all_expired(self, expires: timedelta) -> QuerySet[TaskResult]: ...
    def delete_expired(self, expires: timedelta) -> None: ...

class GroupResultManager(models.Manager[GroupResult]):
    def get_group(self, group_id: str) -> GroupResult: ...
    def store_group_result(
        self,
        content_type: str,
        content_encoding: str,
        group_id: str,
        result: str | None,
        using: str | None = ...,
    ) -> GroupResult: ...



from types import ModuleType
from typing import Any, NamedTuple

from celery import execute as execute
from celery import local as local
from celery import messaging as messaging
from celery._state import current_app as current_app
from celery._state import current_task as current_task
from celery.app import bugreport as bugreport
from celery.app import shared_task
from celery.app.base import Celery
from celery.app.task import Task
from celery.canvas import (
    Signature,
    chain,
    chord,
    chunks,
    group,
    signature,
    xmap,
    xstarmap,
)
from celery.canvas import maybe_signature as maybe_signature
from celery.canvas import signature as subtask
from celery.utils import uuid

# These are lazy module proxies at runtime
log: ModuleType
registry: ModuleType

class version_info_t(NamedTuple):
    major: int
    minor: int
    micro: int
    releaselevel: str
    serial: str

VERSION: version_info_t
version_info: version_info_t
VERSION_BANNER: str
SERIES: str
__version__: str
__author__: str
__contact__: str
__homepage__: str
__docformat__: str

def _find_option_with_arg(
    argv: list[str], short_opts: str | None = ..., long_opts: list[str] | None = ...
) -> str | None: ...
def maybe_patch_concurrency(
    argv: list[str] | None = ...,
    short_opts: str | None = ...,
    long_opts: list[str] | None = ...,
    patches: dict[str, Any] | None = ...,
) -> None: ...

# Note: These are lazy module proxies at runtime
# log, execute, registry, messaging are module-level lazy proxies

__all__ = (
    "SERIES",
    "VERSION",
    "VERSION_BANNER",
    "Celery",
    "Signature",
    "Task",
    "__author__",
    "__contact__",
    "__docformat__",
    "__homepage__",
    "__version__",
    "_find_option_with_arg",
    "bugreport",
    "chain",
    "chord",
    "chunks",
    "current_app",
    "current_task",
    "execute",
    "group",
    "local",
    "log",
    "maybe_patch_concurrency",
    "maybe_signature",
    "messaging",
    "registry",
    "shared_task",
    "signature",
    "subtask",
    "uuid",
    "version_info",
    "version_info_t",
    "xmap",
    "xstarmap",
)



from typing import Any

__all__ = ("Pidbox", "gPidbox")

class Pidbox:
    consumer: Any
    connection: Any
    node: Any

    def __init__(self, c: Any) -> None: ...
    def on_stop(self) -> None: ...
    def start(self, c: Any) -> None: ...
    def stop(self, c: Any) -> None: ...
    def reset(self) -> None: ...
    def shutdown(self, c: Any) -> None: ...
    def on_message(self, body: Any, message: Any) -> None: ...

class gPidbox(Pidbox):
    def __init__(self, c: Any) -> None: ...
    def loop(self, c: Any) -> None: ...



from typing import Any

from kombu.transport.virtual import Channel as VirtualChannel
from kombu.transport.virtual import Transport as VirtualTransport
from kombu.utils.objects import cached_property
from sqlalchemy.engine import Engine
from sqlalchemy.exc import OperationalError
from sqlalchemy.orm import Session, sessionmaker
from typing_extensions import override

VERSION: tuple[int, int, int]

class Channel(VirtualChannel):
    _session: Any | None
    _engines: dict[str, Any]
    queue_tablename: str
    message_tablename: str

    def __init__(self, connection: Any, **kwargs: Any) -> None: ...
    @property
    def session(self) -> Session: ...
    @cached_property
    def queue_cls(self) -> type[Any]: ...
    @cached_property
    def message_cls(self) -> type[Any]: ...
    def _configure_entity_tablenames(self, opts: dict[str, Any]) -> None: ...
    def _engine_from_config(self) -> Engine: ...
    def _open(self) -> tuple[Engine, sessionmaker[Session]]: ...
    @override
    def _get(self, queue: str) -> Any: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
    @override
    def _put(self, queue: str, payload: Any, **kwargs: Any) -> None: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
    @override
    def _size(self, queue: str) -> int: ...

_Channel = Channel

class Transport(VirtualTransport):
    Channel: type[_Channel]  # pyright: ignore[reportIncompatibleVariableOverride]

    can_parse_url: bool
    default_port: int  # pyright: ignore[reportIncompatibleVariableOverride]
    driver_type: str
    driver_name: str
    connection_errors: tuple[type[OperationalError], ...]  # pyright: ignore[reportIncompatibleVariableOverride]

    @override
    def driver_version(self) -> str: ...



from collections.abc import Callable
from typing import Any

from celery.concurrency.base import BasePool
from typing_extensions import override

__all__ = ("TaskPool",)

class TaskPool(BasePool):
    def __init__(self, *args: Any, **kwargs: Any) -> None: ...
    @override
    def on_start(self) -> None: ...
    @override
    def on_stop(self) -> None: ...
    @override
    def on_apply(
        self,
        target: Any,
        args: tuple[Any, ...] | None = None,
        kwargs: dict[str, Any] | None = None,
        callback: Any = None,
        accept_callback: Any = None,
        timeout: float | None = None,
        timeout_callback: Callable[..., Any] | None = None,
        apply_target: Callable[..., Any] = ...,
        **_: Any,
    ) -> Any: ...
    def grow(self, n: int = 1) -> None: ...
    def shrink(self, n: int = 1) -> None: ...



from celery.app.base import Celery

__all__ = ("setup_security",)

def setup_security(
    allowed_serializers: list[str] | None = None,
    key: str | None = None,
    key_password: str | None = None,
    cert: str | None = None,
    store: str | None = None,
    digest: str | None = None,
    serializer: str = "json",
    app: Celery | None = None,
) -> None: ...
def disable_untrusted_serializers(
    whitelist: list[str] | None = None,
) -> None: ...



from collections.abc import Callable, Iterable, Iterator
from contextlib import contextmanager
from operator import itemgetter
from typing import Any

import kombu
from celery.app.base import Celery
from kombu.utils.objects import cached_property

__all__ = ("EventReceiver",)

class EventReceiver:
    Consumer: Callable[..., Any]
    app: Celery | None
    handlers: dict[str, Callable[..., Any]]
    channel_errors: cached_property[tuple[type[Exception], ...]]
    connection_errors: cached_property[tuple[type[Exception], ...]]
    connect_max_retries: int | None
    restart_limit: Any
    should_stop: bool

    def __init__(
        self,
        channel: Any,
        handlers: dict[str, Callable[..., Any]] | None = None,
        routing_key: str = "#",
        node_id: str | None = None,
        app: Celery | None = None,
        queue_prefix: str | None = None,
        accept: Iterable[str] | None = None,
        queue_ttl: float | None = None,
        queue_expires: float | None = None,
        queue_exclusive: bool | None = None,
        queue_durable: bool | None = None,
    ) -> None: ...
    @property
    def connection(self) -> kombu.Connection: ...
    def process(
        self,
        type: str,
        event: dict[str, Any],
    ) -> None: ...
    def capture(
        self,
        limit: int | None = None,
        timeout: float | None = None,
        wakeup: bool = True,
    ) -> None: ...
    def itercapture(
        self,
        limit: int | None = None,
        timeout: float | None = None,
        wakeup: bool = True,
    ) -> Iterator[tuple[str, dict[str, Any]]]: ...
    def consume(
        self,
        limit: int | None = None,
        timeout: float | None = None,
        safety_interval: float = 1,
        **kwargs: Any,
    ) -> None: ...
    def run(
        self,
        _tokens: int = 1,
        **kwargs: Any,
    ) -> None: ...
    @contextmanager
    def consumer_context(self, wakeup: bool = True) -> Iterator[kombu.Consumer]: ...
    def on_consume_ready(
        self,
        connection: Any,
        channel: Any,
        consumers: list[Any],
        wakeup: bool = True,
        **kwargs: Any,
    ) -> None: ...
    def on_consume_end(self, connection: Any, channel: Any) -> None: ...
    def on_connection_revived(self) -> None: ...
    def on_connection_error(self, exc: Exception, interval: float) -> None: ...
    def on_decode_error(self, message: Any, exc: Exception) -> None: ...
    def on_iteration(self) -> None: ...
    def maybe_conn_error(self, fun: Callable[..., Any]) -> Any: ...
    def create_connection(self) -> kombu.Connection: ...
    def establish_connection(self) -> kombu.Connection: ...
    @contextmanager
    def extra_context(
        self, connection: kombu.Connection, channel: Any
    ) -> Iterator[None]: ...
    def event_from_message(
        self,
        body: Any,
        localize: bool = True,
        now: Callable[[], float] = ...,
        tzfields: itemgetter[Any] = ...,
        adjust_timestamp: Callable[..., float] = ...,
        CLIENT_CLOCK_SKEW: int = -1,
    ) -> tuple[str, dict[str, Any]]: ...
    def get_consumers(self, Consumer: type[Any], channel: Any) -> list[Any]: ...
    def wakeup_workers(self, channel: Any = None) -> None: ...



from collections.abc import Callable, Container, Mapping, Sequence
from functools import partial
from typing import Any, TypeVar

import kombu
from celery import Celery
from celery.utils.nodenames import worker_direct
from kombu import Consumer, Message, Producer, Queue

__all__ = (
    "State",
    "StopFiltering",
    "migrate_task",
    "migrate_tasks",
    "move",
    "move_by_idmap",
    "move_by_taskmap",
    "move_direct",
    "move_direct_by_id",
    "move_task_by_id",
    "republish",
    "start_filter",
    "task_id_eq",
    "task_id_in",
)

class StopFiltering(Exception): ...

class State:
    count: int = 0
    filtered: int = 0
    total_apx: int = 0

    @property
    def strtotal(self) -> str: ...

def republish(
    producer: Producer,
    message: Message,
    exchange: float | None = None,
    routing_key: str | None = None,
    remove_props: list[str] | None = None,
) -> None: ...
def migrate_task(
    producer: Producer,
    body_: Mapping[str, Any],
    message: Message,
    queues: Mapping[str, Any] | None = None,
) -> None: ...

_T = TypeVar("_T")

def filter_callback(
    callback: Callable[[Mapping[str, Any], Message], _T],
    tasks: Container[str],
) -> Callable[[Mapping[str, Any], Message], _T | None]: ...
def migrate_tasks(
    source: Sequence[str],
    dest: str | Queue,
    migrate: Callable[..., Any] = ...,
    app: Celery[Any] | None = None,
    queues: str | list[str] | dict[str, str] | None = None,
    **kwargs: Any,
) -> State: ...
def move(
    predicate: Callable[[Any, Any], Any],
    connection: kombu.Connection | None = None,
    exchange: str | kombu.Exchange | None = None,
    routing_key: str | None = None,
    source: Sequence[str] | None = None,
    app: Celery[Any] | None = None,
    callback: Callable[[Any, Any, Any], Any] | None = None,
    limit: int | None = None,
    transform: Callable[[Any], Any] | None = None,
    **kwargs: Any,
) -> State: ...
def expand_dest(
    ret: tuple[str | kombu.Exchange, str],
    exchange: str | kombu.Exchange,
    routing_key: str,
) -> tuple[str | kombu.Exchange, str]: ...
def task_id_eq(task_id: str, body: Mapping[str, Any], message: Message) -> bool: ...
def task_id_in(
    ids: Container[str], body: Mapping[str, Any], message: Message
) -> bool: ...
def prepare_queues(
    queues: str | list[str] | dict[str, str] | None,
) -> dict[str, str]: ...

class Filterer:
    def __init__(
        self,
        app: Celery[Any],
        conn: kombu.Connection,
        filter: Callable[[Any, Any], Any],
        limit: int | None = None,
        timeout: float = 1.0,
        ack_messages: bool = False,
        tasks: str | None = None,
        queues: str | list[str] | dict[str, str] | None = None,
        callback: Callable[..., Any] | None = None,
        forever: bool = False,
        on_declare_queue: Callable[[Queue], None] | None = None,
        consume_from: Container[str] | None = None,
        state: State | None = None,
        accept: Sequence[str] | None = None,
        **kwargs: Any,
    ) -> None: ...
    def start(self) -> State: ...
    def update_state(self, body: Mapping[str, Any], message: Message) -> None: ...
    def ack_message(self, body: Mapping[str, Any], message: Message) -> None: ...
    def create_consumer(self) -> Consumer: ...
    def prepare_consumer(self, consumer: Consumer) -> Consumer: ...
    def declare_queues(self, consumer: Consumer) -> None: ...

def start_filter(
    app: Celery[Any],
    conn: kombu.Connection | None,
    filter: Callable[[Any, Any], Any],
    limit: int | None = None,
    timeout: float = 1.0,
    ack_messages: bool = False,
    tasks: str | None = None,
    queues: str | list[str] | dict[str, str] | None = None,
    callback: Callable[..., Any] | None = None,
    forever: bool = False,
    on_declare_queue: Callable[[Queue], None] | None = None,
    consume_from: Container[str] | None = None,
    state: State | None = None,
    accept: Sequence[str] | None = None,
    **kwargs: Any,
) -> State: ...
def move_task_by_id(task_id: str, dest: str | Queue, **kwargs: Any) -> State: ...
def move_by_idmap(map: Mapping[str, Any], **kwargs: Any) -> State: ...
def move_by_taskmap(map: Mapping[str, Any], **kwargs: Any) -> State: ...
def filter_status(
    state: State, body: Mapping[str, Any], message: Message, **kwargs: Any
) -> None: ...

move_direct = partial(move, transform=worker_direct)
move_direct_by_id = partial(move_task_by_id, transform=worker_direct)
move_direct_by_idmap = partial(move_by_idmap, transform=worker_direct)
move_direct_by_taskmap = partial(move_by_taskmap, transform=worker_direct)



from collections.abc import Callable, Sequence
from datetime import datetime
from typing import (
    Any,
    Concatenate,
    Literal,
    TypeVar,
    overload,
)

from celery.app import control as control
from celery.app import events as events
from celery.app import task as task
from celery.app.base import Celery
from celery.app.task import Context, Task
from celery.local import Proxy
from celery.utils.threads import _LocalStack
from typing_extensions import ParamSpec

__all__ = (
    "AppPickler",
    "Celery",
    "app_or_default",
    "bugreport",
    "default_app",
    "disable_trace",
    "enable_trace",
    "pop_current_task",
    "push_current_task",
    "shared_task",
)

class AppPickler:
    def __call__(self, cls: type[Celery], *args: Any) -> Celery: ...
    def build_kwargs(self, *args: Any) -> dict[str, Any]: ...
    def build_standard_kwargs(
        self,
        main: str | None,
        changes: dict[str, Any] | None,
        loader: Any,
        backend: Any,
        amqp: Any,
        events: Any,  # noqa: F811
        log: Any,
        control: Any,  # noqa: F811
        accept_magic_kwargs: bool,
        config_source: Any | None = None,
    ) -> dict[str, Any]: ...
    def construct(self, cls: type[Celery], **kwargs: Any) -> Celery: ...
    def prepare(self, app: Celery, **kwargs: Any) -> dict[str, Any]: ...

def app_or_default(app: Celery | None = ...) -> Celery: ...
def bugreport(app: Celery | None = ...) -> str: ...
def enable_trace() -> None: ...
def disable_trace() -> None: ...
def push_current_task(obj: Task[Any, Any]) -> None: ...
def pop_current_task() -> Task[Any, Any] | None: ...

default_app: Proxy[Celery]

_T = TypeVar("_T", bound=Task[Any, Any])
_P = ParamSpec("_P")
_R = TypeVar("_R")

@overload
def shared_task(fun: Callable[_P, _R]) -> Task[_P, _R]: ...
@overload
def shared_task(
    *,
    name: str = ...,
    serializer: str = ...,
    bind: Literal[False] = ...,
    autoretry_for: Sequence[type[BaseException]] = ...,
    dont_autoretry_for: Sequence[type[BaseException]] = ...,
    max_retries: int | None = ...,
    default_retry_delay: int = ...,
    acks_late: bool = ...,
    ignore_result: bool = ...,
    soft_time_limit: int = ...,
    time_limit: int = ...,
    base: None = ...,
    retry_kwargs: dict[str, Any] = ...,
    retry_backoff: bool | int = ...,
    retry_backoff_max: int = ...,
    retry_jitter: bool = ...,
    typing: bool = ...,
    rate_limit: str | None = ...,
    trail: bool = ...,
    send_events: bool = ...,
    store_errors_even_if_ignored: bool = ...,
    autoregister: bool = ...,
    track_started: bool = ...,
    acks_on_failure_or_timeout: bool = ...,
    reject_on_worker_lost: bool = ...,
    throws: tuple[type[Exception], ...] = ...,
    expires: float | datetime | None = ...,
    priority: int | None = ...,
    resultrepr_maxsize: int = ...,
    request_stack: _LocalStack[Context] = ...,
    abstract: bool = ...,
    queue: str = ...,
    pydantic: bool = ...,
) -> Callable[[Callable[_P, _R]], Task[_P, _R]]: ...
@overload
def shared_task(
    *,
    name: str = ...,
    serializer: str = ...,
    bind: Literal[True],
    autoretry_for: Sequence[type[BaseException]] = ...,
    dont_autoretry_for: Sequence[type[BaseException]] = ...,
    max_retries: int | None = ...,
    default_retry_delay: int = ...,
    acks_late: bool = ...,
    ignore_result: bool = ...,
    soft_time_limit: int = ...,
    time_limit: int = ...,
    base: None = ...,
    retry_kwargs: dict[str, Any] = ...,
    retry_backoff: bool | int = ...,
    retry_backoff_max: int = ...,
    retry_jitter: bool = ...,
    typing: bool = ...,
    rate_limit: str | None = ...,
    trail: bool = ...,
    send_events: bool = ...,
    store_errors_even_if_ignored: bool = ...,
    autoregister: bool = ...,
    track_started: bool = ...,
    acks_on_failure_or_timeout: bool = ...,
    reject_on_worker_lost: bool = ...,
    throws: tuple[type[Exception], ...] = ...,
    expires: float | datetime | None = ...,
    priority: int | None = ...,
    resultrepr_maxsize: int = ...,
    request_stack: _LocalStack[Context] = ...,
    abstract: bool = ...,
    queue: str = ...,
    pydantic: bool = ...,
) -> Callable[[Callable[Concatenate[Task[_P, _R], _P], _R]], Task[_P, _R]]: ...
@overload
def shared_task(
    *,
    name: str = ...,
    serializer: str = ...,
    bind: Literal[True],
    autoretry_for: Sequence[type[BaseException]] = ...,
    dont_autoretry_for: Sequence[type[BaseException]] = ...,
    max_retries: int | None = ...,
    default_retry_delay: int = ...,
    acks_late: bool = ...,
    ignore_result: bool = ...,
    soft_time_limit: int = ...,
    time_limit: int = ...,
    base: type[_T],
    retry_kwargs: dict[str, Any] = ...,
    retry_backoff: bool | int = ...,
    retry_backoff_max: int = ...,
    retry_jitter: bool = ...,
    typing: bool = ...,
    rate_limit: str | None = ...,
    trail: bool = ...,
    send_events: bool = ...,
    store_errors_even_if_ignored: bool = ...,
    autoregister: bool = ...,
    track_started: bool = ...,
    acks_on_failure_or_timeout: bool = ...,
    reject_on_worker_lost: bool = ...,
    throws: tuple[type[Exception], ...] = ...,
    expires: float | datetime | None = ...,
    priority: int | None = ...,
    resultrepr_maxsize: int = ...,
    request_stack: _LocalStack[Context] = ...,
    abstract: bool = ...,
    queue: str = ...,
    **options: Any,
) -> Callable[
    [
        Callable[
            Concatenate[_T, _P],
            _R,
        ]
        | Callable[
            [],
            _R,
        ]
    ],
    _T,
]: ...
@overload
def shared_task(
    *,
    name: str = ...,
    serializer: str = ...,
    bind: bool = ...,
    autoretry_for: Sequence[type[BaseException]] = ...,
    dont_autoretry_for: Sequence[type[BaseException]] = ...,
    max_retries: int | None = ...,
    default_retry_delay: int = ...,
    acks_late: bool = ...,
    ignore_result: bool = ...,
    soft_time_limit: int = ...,
    time_limit: int = ...,
    base: type[_T],
    retry_kwargs: dict[str, Any] = ...,
    retry_backoff: bool | int = ...,
    retry_backoff_max: int = ...,
    retry_jitter: bool = ...,
    typing: bool = ...,
    rate_limit: str | None = ...,
    trail: bool = ...,
    send_events: bool = ...,
    store_errors_even_if_ignored: bool = ...,
    autoregister: bool = ...,
    track_started: bool = ...,
    acks_on_failure_or_timeout: bool = ...,
    reject_on_worker_lost: bool = ...,
    throws: tuple[type[Exception], ...] = ...,
    expires: float | datetime | None = ...,
    priority: int | None = ...,
    resultrepr_maxsize: int = ...,
    request_stack: _LocalStack[Context] = ...,
    abstract: bool = ...,
    queue: str = ...,
    **options: Any,
) -> Callable[[Callable[..., Any]], _T]: ...



from typing import Any

from kombu.message import Message

class BaseAsyncMessage(Message): ...
class AsyncRawMessage(BaseAsyncMessage): ...

class AsyncMessage(BaseAsyncMessage):
    def encode(self, value: str) -> str: ...
    def __getitem__(self, item: str) -> Any: ...



from typing import Any, Generic, ParamSpec, TypeVar

from celery import Task
from celery.result import AsyncResult
from typing_extensions import override

__all__ = ("AbortableAsyncResult", "AbortableTask")

ABORTED: str = "ABORTED"

_R_co = TypeVar("_R_co", covariant=True)
_P = ParamSpec("_P")

class AbortableAsyncResult(AsyncResult[_R_co], Generic[_R_co]):
    def is_aborted(self) -> bool: ...
    def abort(self) -> Any: ...

class AbortableTask(Task[_P, _R_co], Generic[_P, _R_co]):
    abstract: bool = True

    @override
    def AsyncResult(self, task_id: str) -> AbortableAsyncResult[_R_co]: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
    def is_aborted(self, *, task_id: str, **kwargs: Any) -> bool: ...



import datetime
from collections import defaultdict
from collections.abc import Callable, Sequence
from types import TracebackType
from typing import (
    Any,
    Concatenate,
    Generic,
    Literal,
    NoReturn,
    overload,
)

__all__ = ("Celery",)

import celery.app
import celery.result
import kombu
from celery.app.amqp import AMQP
from celery.app.control import Control
from celery.app.events import Events
from celery.app.log import Logging
from celery.app.registry import TaskRegistry
from celery.app.routes import Router
from celery.app.task import Context
from celery.app.task import Task as CeleryTask
from celery.app.utils import Settings
from celery.apps.beat import Beat as CeleryBeat
from celery.apps.worker import Worker as CeleryWorker
from celery.backends.base import Backend
from celery.canvas import Signature, chord
from celery.loaders.base import BaseLoader
from celery.schedules import BaseSchedule
from celery.utils.dispatch import Signal
from celery.utils.objects import FallbackContext
from celery.utils.threads import _LocalStack
from celery.worker import WorkController as CeleryWorkController
from typing_extensions import ParamSpec, Self, TypeVar

_T = TypeVar("_T", bound=CeleryTask[Any, Any])
_T_1 = TypeVar("_T_1")
_T_Global = TypeVar(
    "_T_Global",
    bound=CeleryTask[Any, Any],
    default=CeleryTask[Any, Any],
)
_P = ParamSpec("_P")
_R = TypeVar("_R")

class Celery(Generic[_T_Global]):
    # Class-level constants
    IS_WINDOWS: bool
    IS_macOS: bool
    SYSTEM: str
    Pickler: type[Any]

    # Configuration class names
    amqp_cls: str | None
    backend_cls: str | None
    control_cls: str | None
    events_cls: str | None
    loader_cls: str | None
    log_cls: str | None
    registry_cls: str | None
    task_cls: str | None

    # Instance attributes (None at class level)
    main: str | None
    steps: defaultdict[str, set[Any]] | None
    user_options: dict[str, Any] | None
    builtin_fixups: set[str]

    # Signals (always set on instances in __init__)
    on_configure: Signal
    on_after_configure: Signal
    on_after_finalize: Signal
    on_after_fork: Signal

    # Reduce methods for pickling
    def __reduce_args__(self) -> tuple[Any, ...]: ...
    def __reduce_keys__(self) -> dict[str, Any]: ...
    def __reduce_v1__(self) -> tuple[Any, ...]: ...
    def __init__(
        self,
        main: str | None = None,
        loader: Any | None = None,
        backend: str | type[Backend] | None = None,
        amqp: str | type[AMQP] | None = None,
        events: str | type[celery.app.events.Events] | None = None,
        log: str | type[Logging] | None = None,
        control: str | type[celery.app.control.Control] | None = None,
        set_as_current: bool = True,
        tasks: str | type[TaskRegistry] | None = None,
        broker: str | None = None,
        include: list[str] | tuple[str, ...] | None = None,
        changes: dict[str, Any] | None = None,
        config_source: str | object | None = None,
        fixups: list[str] | None = None,
        task_cls: str | type[_T_Global] | None = None,
        autofinalize: bool = True,
        namespace: str | None = None,
        strict_typing: bool = True,
        *,
        # Config kwargs that can be passed directly (captured by **kwargs at runtime)
        imports: list[str] | tuple[str, ...] | None = ...,
        broker_connection_retry: bool = ...,
        broker_connection_max_retries: int = ...,
        broker_channel_error_retry: bool = ...,
        broker_login_method: str = ...,
        broker_transport_options: dict[str, Any] = ...,
        broker_connection_retry_on_startup: bool = ...,
        broker_connection_timeout: float = ...,
        result_backend_transport_options: dict[str, Any] | None = ...,
        result_extended: bool = ...,
        result_expires: datetime.timedelta = ...,
        beat_schedule: dict[str, Any] | None = ...,
        task_queues: list[kombu.Queue] | None = ...,
        task_default_queue: str = ...,
        task_default_exchange: str = ...,
        task_default_exchange_type: str = ...,
        task_default_routing_key: str = ...,
        task_default_delivery_mode: str = ...,
        task_create_missing_queues: bool = ...,
        task_routes: dict[Any, Any] | Sequence[Any] = ...,
        task_acks_late: bool = ...,
        task_time_limit: int = ...,
        task_soft_time_limit: int = ...,
        task_track_started: bool = ...,
        task_serializer: str = ...,
        worker_prefetch_multiplier: int = ...,
        worker_max_tasks_per_child: int = ...,
        worker_concurrency: int = ...,
        worker_max_memory_per_child: int = ...,
        worker_disable_rate_limits: bool = ...,
        worker_cancel_long_running_tasks_on_connection_loss: bool = ...,
        worker_hijack_root_logger: bool = ...,
        worker_log_format: str = ...,
        worker_task_log_format: str = ...,
        worker_redirect_stdouts: bool = ...,
        worker_redirect_stdouts_level: str = ...,
        **kwargs: Any,
    ) -> None: ...
    def _task_from_fun(
        self,
        fun: Callable[_P, _R],
        name: str | None = None,
        base: type[_T_Global] | None = None,
        bind: bool = False,
        # pydantic options (new in celery 5.4+)
        pydantic: bool = False,
        pydantic_strict: bool = False,
        pydantic_context: dict[str, Any] | None = None,
        pydantic_dump_kwargs: dict[str, Any] | None = None,
        *,
        # autoretry options (captured by **options at runtime)
        autoretry_for: Sequence[type[BaseException]] = ...,
        dont_autoretry_for: Sequence[type[BaseException]] = ...,
        retry_kwargs: dict[str, Any] = ...,
        retry_backoff: bool | int = ...,
        retry_backoff_max: int = ...,
        retry_jitter: bool = ...,
        # task options
        typing: bool = ...,
        max_retries: int | None = ...,
        default_retry_delay: int = ...,
        rate_limit: str | None = ...,
        ignore_result: bool = ...,
        trail: bool = ...,
        send_events: bool = ...,
        store_errors_even_if_ignored: bool = ...,
        serializer: str = ...,
        time_limit: int | None = ...,
        soft_time_limit: int | None = ...,
        autoregister: bool = ...,
        track_started: bool = ...,
        acks_late: bool = ...,
        acks_on_failure_or_timeout: bool = ...,
        reject_on_worker_lost: bool = ...,
        throws: tuple[type[Exception], ...] = ...,
        expires: float | datetime.datetime | None = ...,
        priority: int | None = ...,
        resultrepr_maxsize: int = ...,
        request_stack: _LocalStack[Context] = ...,
        abstract: bool = ...,
        after_return: Callable[..., Any] = ...,
        on_retry: Callable[..., Any] = ...,
        **options: Any,
    ) -> _T_Global: ...
    def on_init(self) -> None: ...
    def set_current(self) -> None: ...
    def set_default(self) -> None: ...
    def close(self) -> None: ...
    def start(self, argv: list[str] | None = None) -> NoReturn: ...
    def worker_main(self, argv: list[str] | None = None) -> NoReturn: ...
    @overload
    def task(self, fun: Callable[_P, _R]) -> _T_Global: ...
    @overload
    def task(
        self,
        *,
        name: str = ...,
        serializer: str = ...,
        bind: bool = ...,
        autoretry_for: Sequence[type[BaseException]] = ...,
        dont_autoretry_for: Sequence[type[BaseException]] = ...,
        max_retries: int | None = ...,
        default_retry_delay: int = ...,
        acks_late: bool = ...,
        ignore_result: bool = ...,
        soft_time_limit: int = ...,
        time_limit: int = ...,
        base: type[_T],
        retry_kwargs: dict[str, Any] = ...,
        retry_backoff: bool | int = ...,
        retry_backoff_max: int = ...,
        retry_jitter: bool = ...,
        typing: bool = ...,
        rate_limit: str | None = ...,
        trail: bool = ...,
        send_events: bool = ...,
        store_errors_even_if_ignored: bool = ...,
        autoregister: bool = ...,
        track_started: bool = ...,
        acks_on_failure_or_timeout: bool = ...,
        reject_on_worker_lost: bool = ...,
        throws: tuple[type[Exception], ...] = ...,
        expires: float | datetime.datetime | None = ...,
        priority: int | None = ...,
        resultrepr_maxsize: int = ...,
        request_stack: _LocalStack[Context] = ...,
        abstract: bool = ...,
        queue: str = ...,
        after_return: Callable[..., Any] = ...,
        on_retry: Callable[..., Any] = ...,
        **options: Any,
    ) -> Callable[[Callable[..., Any]], _T]: ...
    @overload
    def task(
        self,
        *,
        name: str = ...,
        serializer: str = ...,
        bind: Literal[False] = ...,
        autoretry_for: Sequence[type[BaseException]] = ...,
        dont_autoretry_for: Sequence[type[BaseException]] = ...,
        max_retries: int | None = ...,
        default_retry_delay: int = ...,
        acks_late: bool = ...,
        ignore_result: bool = ...,
        soft_time_limit: int = ...,
        time_limit: int = ...,
        base: None = ...,
        retry_kwargs: dict[str, Any] = ...,
        retry_backoff: bool | int = ...,
        retry_backoff_max: int = ...,
        retry_jitter: bool = ...,
        typing: bool = ...,
        rate_limit: str | None = ...,
        trail: bool = ...,
        send_events: bool = ...,
        store_errors_even_if_ignored: bool = ...,
        autoregister: bool = ...,
        track_started: bool = ...,
        acks_on_failure_or_timeout: bool = ...,
        reject_on_worker_lost: bool = ...,
        throws: tuple[type[Exception], ...] = ...,
        expires: float | datetime.datetime | None = ...,
        priority: int | None = ...,
        resultrepr_maxsize: int = ...,
        request_stack: _LocalStack[Context] = ...,
        abstract: bool = ...,
        queue: str = ...,
        after_return: Callable[..., Any] = ...,
        on_retry: Callable[..., Any] = ...,
        **options: Any,
    ) -> Callable[[Callable[_P, _R]], _T_Global]: ...
    @overload
    def task(
        self,
        *,
        name: str = ...,
        serializer: str = ...,
        bind: Literal[True],
        autoretry_for: Sequence[type[BaseException]] = ...,
        dont_autoretry_for: Sequence[type[BaseException]] = ...,
        max_retries: int | None = ...,
        default_retry_delay: int = ...,
        acks_late: bool = ...,
        ignore_result: bool = ...,
        soft_time_limit: int = ...,
        time_limit: int = ...,
        base: None = ...,
        retry_kwargs: dict[str, Any] = ...,
        retry_backoff: bool | int = ...,
        retry_backoff_max: int = ...,
        retry_jitter: bool = ...,
        typing: bool = ...,
        rate_limit: str | None = ...,
        trail: bool = ...,
        send_events: bool = ...,
        store_errors_even_if_ignored: bool = ...,
        autoregister: bool = ...,
        track_started: bool = ...,
        acks_on_failure_or_timeout: bool = ...,
        reject_on_worker_lost: bool = ...,
        throws: tuple[type[Exception], ...] = ...,
        expires: float | datetime.datetime | None = ...,
        priority: int | None = ...,
        resultrepr_maxsize: int = ...,
        request_stack: _LocalStack[Context] = ...,
        abstract: bool = ...,
        queue: str = ...,
        after_return: Callable[..., Any] = ...,
        on_retry: Callable[..., Any] = ...,
        **options: Any,
    ) -> Callable[[Callable[Concatenate[_T_Global, _P], _R]], _T_Global]: ...
    def type_checker(
        self, fun: Callable[_P, _T_1], bound: bool = False
    ) -> Callable[_P, _T_1]: ...
    def register_task(
        self,
        task: _T | type[_T],
        *,
        autoretry_for: Sequence[type[BaseException]] = ...,
        dont_autoretry_for: Sequence[type[BaseException]] = ...,
        retry_kwargs: dict[str, Any] = ...,
        retry_backoff: bool | int = ...,
        retry_backoff_max: int = ...,
        retry_jitter: bool = ...,
    ) -> _T: ...
    def gen_task_name(self, name: str, module: str) -> str: ...
    def finalize(self, auto: bool = False) -> None: ...
    def add_defaults(self, fun: Callable[[], dict[str, Any]]) -> None: ...
    def config_from_object(
        self,
        obj: Any,
        silent: bool = False,
        force: bool = False,
        namespace: str | None = None,
    ) -> Settings: ...
    def config_from_envvar(
        self, variable_name: str, silent: bool = False, force: bool = False
    ) -> None: ...
    def config_from_cmdline(
        self, argv: list[str], namespace: str = "celery"
    ) -> None: ...
    def setup_security(
        self,
        allowed_serializers: set[str] | None = None,
        key: str | None = None,
        key_password: str | None = None,
        cert: str | None = None,
        store: str | None = None,
        digest: str = "sha256",
        serializer: str = "json",
    ) -> None: ...
    def autodiscover_tasks(
        self,
        packages: list[str] | Callable[[], list[str]] | None = None,
        related_name: str = "tasks",
        force: bool = False,
    ) -> None: ...
    def send_task(
        self,
        name: str,
        args: Sequence[Any] | None = None,
        kwargs: dict[str, Any] | None = None,
        countdown: float | None = None,
        eta: datetime.datetime | None = None,
        task_id: str | None = None,
        producer: kombu.Producer | None = None,
        connection: kombu.Connection | None = None,
        router: Router | None = None,
        result_cls: type[celery.result.AsyncResult[Any]] | None = None,
        expires: float | datetime.datetime | None = None,
        publisher: kombu.Producer | None = None,
        link: Signature[Any] | None = None,
        link_error: Signature[Any] | None = None,
        add_to_parent: bool = True,
        group_id: str | None = None,
        group_index: int | None = None,
        retries: int = 0,
        chord: chord | None = None,
        reply_to: str | None = None,
        time_limit: int | None = None,
        soft_time_limit: int | None = None,
        root_id: str | None = None,
        parent_id: str | None = None,
        route_name: str | None = None,
        shadow: str | None = None,
        chain: Any | None = None,
        task_type: Any | None = None,
        replaced_task_nesting: int = 0,
        **options: Any,
    ) -> celery.result.AsyncResult[Any]: ...
    def connection_for_read(
        self, url: str | None = None, **kwargs: Any
    ) -> kombu.Connection: ...
    def connection_for_write(
        self, url: str | None = None, **kwargs: Any
    ) -> kombu.Connection: ...
    def connection(
        self,
        hostname: str | None = None,
        userid: str | None = None,
        password: str | None = None,
        virtual_host: str | None = None,
        port: int | None = None,
        ssl: bool | dict[str, Any] | None = None,
        connect_timeout: int | None = None,
        transport: str | None = None,
        transport_options: dict[str, Any] | None = None,
        heartbeat: int | None = None,
        login_method: int | None = None,
        failover_strategy: str | Callable[[], Any] | None = None,
        **kwargs: Any,
    ) -> kombu.Connection: ...

    broker_connection = connection

    def connection_or_acquire(
        self,
        connection: kombu.Connection | None = None,
        pool: bool = True,
        *_: Any,
        **__: Any,
    ) -> FallbackContext[Any, Any]: ...

    default_connection = connection_or_acquire

    def producer_or_acquire(
        self, producer: kombu.Producer | None = None
    ) -> FallbackContext[Any, Any]: ...

    default_producer = producer_or_acquire

    def prepare_config(self, c: Settings) -> Settings: ...
    def now(self) -> datetime.datetime: ...
    def select_queues(self, queues: Sequence[str] | None = None) -> None: ...
    def either(self, default_key: str, *defaults: Any) -> Any: ...
    def bugreport(self) -> str: ...
    def signature(self, *args: Any, **kwargs: Any) -> Signature[Any]: ...
    def add_periodic_task(
        self,
        schedule: BaseSchedule | float,
        sig: Signature[Any],
        args: tuple[Any, ...] = (),
        kwargs: tuple[()] = (),  # runtime uses empty tuple
        name: str | None = None,
        **opts: Any,
    ) -> str: ...
    def create_task_cls(self) -> type[Any]: ...
    def subclass_with_self(
        self,
        Class: type[Any],
        name: str | None = None,
        attribute: str = "app",
        reverse: str | None = None,
        keep_reduce: bool = False,
        **kw: Any,
    ) -> type[Any]: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self,
        typ: type[BaseException] | None,
        exc: BaseException | None,
        tb: TracebackType | None,
    ) -> None: ...
    @property
    def Worker(self) -> type[CeleryWorker]: ...
    @property
    def WorkController(self) -> type[CeleryWorkController]: ...
    @property
    def Beat(self) -> type[CeleryBeat]: ...
    @property
    def Task(self) -> type[_T_Global]: ...
    @property
    def annotations(self) -> list[dict[str, Any]]: ...
    @property
    def AsyncResult(self) -> type[celery.result.AsyncResult[Any]]: ...
    @property
    def ResultSet(self) -> type[celery.result.ResultSet]: ...
    @property
    def GroupResult(self) -> type[celery.result.GroupResult]: ...
    @property
    def pool(self) -> kombu.pools.ProducerPool: ...
    @property
    def current_task(self) -> _T_Global | None: ...
    @property
    def current_worker_task(self) -> _T_Global | None: ...
    @property
    def oid(self) -> str: ...
    @property
    def thread_oid(self) -> str: ...
    @property
    def amqp(self) -> AMQP: ...
    @property
    def backend(self) -> Backend: ...
    @property
    def conf(self) -> Settings: ...
    @conf.setter
    def conf(self, d: Any) -> None: ...
    @property
    def control(self) -> Control: ...
    @property
    def events(self) -> Events: ...
    @property
    def loader(self) -> BaseLoader: ...
    @property
    def log(self) -> Logging: ...
    @property
    def tasks(self) -> TaskRegistry: ...
    @property
    def producer_pool(self) -> kombu.pools.ProducerPool: ...
    def uses_utc_timezone(self) -> bool: ...
    @property
    def timezone(self) -> datetime.timezone: ...



import builtins
from collections.abc import Callable, Generator, Sequence
from contextlib import contextmanager
from typing import Any, TypeAlias

from kombu.connection import Connection
from kombu.entity import Exchange, Queue
from kombu.message import Message
from kombu.messaging import Consumer, Producer
from kombu.transport.base import StdChannel
from kombu.utils.objects import cached_property

__all__ = ("Mailbox", "Node")

_ConsumerType: TypeAlias = Consumer

class Node:
    hostname: str | None
    state: Any | None
    channel: StdChannel | None
    handlers: dict[str, Callable[..., Any]] | None
    mailbox: Mailbox | None

    def __init__(
        self,
        hostname: str | None,
        state: Any | None = ...,
        channel: StdChannel | None = ...,
        handlers: dict[str, Callable[..., Any]] | None = ...,
        mailbox: Mailbox | None = ...,
    ) -> None: ...
    def Consumer(
        self,
        channel: StdChannel | None = ...,
        no_ack: bool = ...,
        accept: Sequence[str] | None = ...,
        **options: Any,
    ) -> _ConsumerType: ...
    def handler(self, fun: Callable[..., Any]) -> Callable[..., Any]: ...
    def listen(
        self,
        channel: StdChannel | None = ...,
        callback: Callable[..., Any] | None = ...,
    ) -> _ConsumerType: ...
    def dispatch(
        self,
        method: str,
        arguments: dict[str, Any] | None = ...,
        reply_to: dict[str, Any] | None = ...,
        ticket: str | None = ...,
        **kwargs: Any,
    ) -> Any: ...
    def dispatch_from_message(
        self, body: Any, message: Message | None = ...
    ) -> None: ...
    def handle_message(self, body: Any, message: Message | None = ...) -> None: ...
    def handle(self, method: str, arguments: dict[str, Any] | None = ...) -> Any: ...
    def handle_call(self, method: str, arguments: dict[str, Any] | None) -> Any: ...
    def handle_cast(self, method: str, arguments: dict[str, Any] | None) -> Any: ...
    def reply(
        self,
        data: Any,
        exchange: Exchange | str,
        routing_key: str,
        ticket: str | None,
        **kwargs: Any,
    ) -> None: ...
    def on_decode_error(self, message: Message, exc: Exception) -> None: ...

class Mailbox:
    namespace: str | None
    connection: Connection | None
    type: str
    exchange: Exchange | None
    exchange_fmt: str
    reply_exchange: Exchange | None
    reply_exchange_fmt: str
    accept: list[str]
    serializer: str | None
    producer_pool: Any | None
    queue_ttl: float | None
    queue_expires: float | None
    queue_durable: bool
    queue_exclusive: bool
    reply_queue_ttl: float | None
    reply_queue_expires: float | None
    node_cls: builtins.type[Node]  # ty: ignore[invalid-type-form]

    def __init__(
        self,
        namespace: str,
        type: str = ...,
        connection: Connection | None = ...,
        clock: Any | None = ...,
        accept: Sequence[str] | None = ...,
        serializer: str | None = ...,
        producer_pool: Any | None = ...,
        queue_ttl: float | None = ...,
        queue_expires: float | None = ...,
        queue_durable: bool = ...,
        queue_exclusive: bool = ...,
        reply_queue_ttl: float | None = ...,
        reply_queue_expires: float | None = ...,
    ) -> None: ...
    def __call__(self, connection: Connection) -> Node: ...  # ty: ignore[invalid-type-form]
    def Node(
        self,
        hostname: str | None = ...,
        state: Any | None = ...,
        channel: StdChannel | None = ...,
        handlers: dict[str, Callable[..., Any]] | None = ...,
    ) -> Node: ...  # ty: ignore[invalid-type-form]
    def get_queue(self, hostname: str) -> Queue: ...
    def get_reply_queue(self) -> Queue: ...
    @contextmanager
    def producer_or_acquire(
        self, producer: Producer | None = ..., channel: StdChannel | None = ...
    ) -> Generator[Producer, None, None]: ...
    def call(
        self,
        destination: Sequence[str],
        command: str,
        kwargs: dict[str, Any] | None = ...,
        timeout: float | None = ...,
        callback: Callable[..., Any] | None = ...,
        channel: StdChannel | None = ...,
    ) -> list[Any] | None: ...
    def cast(
        self,
        destination: Sequence[str],
        command: str,
        kwargs: dict[str, Any] | None = ...,
    ) -> None: ...
    def abcast(self, command: str, kwargs: dict[str, Any] | None = ...) -> None: ...
    def multi_call(
        self,
        command: str,
        kwargs: dict[str, Any] | None = ...,
        timeout: float = ...,
        limit: int | None = ...,
        callback: Callable[..., Any] | None = ...,
        channel: StdChannel | None = ...,
    ) -> list[Any] | None: ...
    def _get_exchange(self, namespace: str, type: str) -> Exchange: ...
    def _get_reply_exchange(self, namespace: str) -> Exchange: ...
    def _broadcast(
        self,
        command: str,
        arguments: dict[str, Any] | None = ...,
        destination: Sequence[str] | None = ...,
        reply: bool = ...,
        timeout: float = ...,
        limit: int | None = ...,
        callback: Callable[..., Any] | None = ...,
        channel: StdChannel | None = ...,
        serializer: str | None = ...,
        pattern: str | None = ...,
        matcher: str | None = ...,
    ) -> list[Any] | None: ...
    def _collect(
        self,
        ticket: str,
        limit: int | None = ...,
        timeout: float = ...,
        callback: Callable[..., Any] | None = ...,
        channel: StdChannel | None = ...,
        accept: Sequence[str] | None = ...,
    ) -> list[Any]: ...
    def _publish(
        self,
        type: str,
        arguments: dict[str, Any],
        destination: Sequence[str] | None = ...,
        reply_ticket: str | None = ...,
        channel: StdChannel | None = ...,
        timeout: float | None = ...,
        serializer: str | None = ...,
        producer: Producer | None = ...,
        pattern: str | None = ...,
        matcher: str | None = ...,
    ) -> None: ...
    def _publish_reply(
        self,
        reply: Any,
        exchange: str | Exchange,
        routing_key: str,
        ticket: str,
        channel: StdChannel | None = ...,
        producer: Producer | None = ...,
        **opts: Any,
    ) -> None: ...
    @property
    def oid(self) -> str: ...
    @cached_property
    def reply_queue(self) -> Queue: ...



from logging import Logger
from typing import Any, TextIO

from kombu.utils.objects import cached_property
from typing_extensions import override

__all__ = ("LOG_LEVELS", "LogMixin", "get_loglevel", "setup_logging")

LOG_LEVELS: dict[str | int, int | str]
DISABLE_TRACEBACKS: str | None

class LogMixin:
    def debug(self, *args: Any, **kwargs: Any) -> None: ...
    def info(self, *args: Any, **kwargs: Any) -> None: ...
    def warn(self, *args: Any, **kwargs: Any) -> None: ...
    def error(self, *args: Any, **kwargs: Any) -> None: ...
    def critical(self, *args: Any, **kwargs: Any) -> None: ...
    def annotate(self, text: str) -> str: ...
    def log(self, severity: int, *args: Any, **kwargs: Any) -> None: ...
    def get_logger(self) -> Logger: ...
    def is_enabled_for(self, level: int | str) -> bool: ...
    def get_loglevel(self, level: int | str) -> int: ...
    @cached_property
    def logger(self) -> Logger: ...
    @property
    def logger_name(self) -> str: ...

class Log(LogMixin):
    _logger_name: str
    _logger: Logger | None

    def __init__(self, name: str, logger: Logger | None = ...) -> None: ...
    @override
    def get_logger(self) -> Logger: ...
    @property
    @override
    def logger_name(self) -> str: ...

def get_loglevel(level: str | int) -> int: ...
def setup_logging(
    loglevel: int | str | None = ..., logfile: str | TextIO | None = ...
) -> Logger: ...
def get_logger(logger: str | Logger) -> Logger: ...
def naive_format_parts(fmt: str) -> Any: ...
def safeify_format(
    fmt: str, args: tuple[Any, ...], filters: dict[str, Any] | None = ...
) -> Any: ...



from threading import Lock
from typing import Any

from typing_extensions import Self, override

__all__ = ("LamportClock", "timetuple")

class timetuple(tuple[int | None, float, str, Any]):
    __slots__ = ()

    def __new__(
        cls, clock: int | None, timestamp: float, id: str, obj: Any = ...
    ) -> Self: ...
    def __getnewargs__(self) -> tuple[int | None, float, str, Any]: ...
    @override
    def __lt__(self, other: tuple[Any, ...]) -> bool: ...
    @override
    def __gt__(self, other: tuple[Any, ...]) -> bool: ...
    @override
    def __le__(self, other: tuple[Any, ...]) -> bool: ...
    @override
    def __ge__(self, other: tuple[Any, ...]) -> bool: ...
    @property
    def clock(self) -> int | None: ...
    @property
    def timestamp(self) -> float: ...
    @property
    def id(self) -> str: ...
    @property
    def obj(self) -> Any: ...

class LamportClock:
    value: int
    mutex: Lock

    def __init__(self, initial_value: int = ..., Lock: type[Lock] = ...) -> None: ...
    def adjust(self, other: int) -> int: ...
    def forward(self) -> int: ...
    def sort_heap(self, h: list[tuple[int, str]]) -> tuple[int, str]: ...



from abc import ABC, ABCMeta, abstractmethod
from typing import Any

__all__ = ("CallableSignature", "CallableTask")

class CallableTask(metaclass=ABCMeta):
    __required_attributes__: frozenset[str]

    @abstractmethod
    def apply(self, *args: Any, **kwargs: Any) -> Any: ...
    @abstractmethod
    def apply_async(self, *args: Any, **kwargs: Any) -> Any: ...
    @abstractmethod
    def delay(self, *args: Any, **kwargs: Any) -> Any: ...
    @classmethod
    def register(cls, other: type) -> type: ...

class CallableSignature(CallableTask, ABC):
    __required_attributes__: frozenset[str]

    @property
    @abstractmethod
    def app(self) -> Any: ...
    @property
    @abstractmethod
    def args(self) -> tuple[Any, ...]: ...
    @property
    @abstractmethod
    def chord_size(self) -> int | None: ...
    @property
    @abstractmethod
    def id(self) -> str | None: ...
    @property
    @abstractmethod
    def immutable(self) -> bool: ...
    @property
    @abstractmethod
    def kwargs(self) -> dict[str, Any]: ...
    @property
    @abstractmethod
    def name(self) -> str | None: ...
    @property
    @abstractmethod
    def options(self) -> dict[str, Any]: ...
    @property
    @abstractmethod
    def subtask_type(self) -> str | None: ...
    @property
    @abstractmethod
    def task(self) -> str | None: ...
    @property
    @abstractmethod
    def type(self) -> Any: ...
    @abstractmethod
    def __invert__(self) -> Any: ...
    @abstractmethod
    def __or__(self, other: Any) -> Any: ...
    @abstractmethod
    def clone(
        self, args: tuple[Any, ...] | None = None, kwargs: dict[str, Any] | None = None
    ) -> Any: ...
    @abstractmethod
    def freeze(
        self,
        id: str | None = None,
        group_id: str | None = None,
        chord: Any | None = None,
        root_id: str | None = None,
        group_index: int | None = None,
    ) -> Any: ...
    @abstractmethod
    def link(self, callback: Any) -> Any: ...
    @abstractmethod
    def link_error(self, errback: Any) -> Any: ...
    @abstractmethod
    def set(self, immutable: bool | None = None, **options: Any) -> Any: ...



from kombu.asynchronous.http.base import Headers as Headers
from kombu.asynchronous.http.base import Request as Request
from kombu.asynchronous.http.base import Response as Response
from kombu.asynchronous.http.curl import CurlClient
from kombu.asynchronous.hub import Hub

__all__ = ("Client", "Headers", "Request", "Response")

def Client(hub: Hub | None = ..., **kwargs: int) -> CurlClient: ...
def get_client(hub: Hub | None = ..., **kwargs: int) -> CurlClient: ...



from collections.abc import Callable, Sequence
from typing import Any

from celery import bootsteps
from celery.worker.components import Hub, Pool, Timer

__all__ = ("WorkController",)

class WorkController:
    class Blueprint(bootsteps.Blueprint): ...
    app: Any
    blueprint: bootsteps.Blueprint | None
    exitcode: int | None
    hub: Hub | None
    pidlock: Any
    pool: Pool | None
    semaphore: Any
    timer: Timer | None

    def rusage(self) -> Any: ...
    @property
    def state(self) -> str: ...
    def __init__(
        self, app: Any = None, hostname: str | None = None, **kwargs: Any
    ) -> None: ...
    def info(self) -> dict[str, Any]: ...
    def on_after_init(self, **kwargs: Any) -> None: ...
    def on_before_init(self, **kwargs: Any) -> None: ...
    def on_close(self) -> None: ...
    def on_consumer_ready(self, consumer: Any) -> None: ...
    def on_init_blueprint(self) -> None: ...
    def on_start(self) -> None: ...
    def on_stopped(self) -> None: ...
    def prepare_args(self, **kwargs: Any) -> dict[str, Any]: ...
    def register_with_event_loop(self, hub: Any) -> None: ...
    def reload(
        self,
        modules: Sequence[str] | None = None,
        reload: bool = False,
        reloader: Any | None = None,
    ) -> None: ...
    def setup_defaults(
        self,
        concurrency: int | None = None,
        loglevel: str | int = "WARN",
        logfile: str | None = None,
        task_events: bool | None = None,
        pool: Any | None = None,
        consumer_cls: Any | None = None,
        timer_cls: Any | None = None,
        timer_precision: float | None = None,
        autoscaler_cls: Any | None = None,
        pool_putlocks: bool | None = None,
        pool_restarts: bool | None = None,
        optimization: str | None = None,
        O: str | None = None,
        statedb: str | None = None,
        time_limit: float | None = None,
        soft_time_limit: float | None = None,
        scheduler: str | None = None,
        pool_cls: Any | None = None,
        state_db: str | None = None,
        task_time_limit: float | None = None,
        task_soft_time_limit: float | None = None,
        scheduler_cls: Any | None = None,
        schedule_filename: str | None = None,
        max_tasks_per_child: int | None = None,
        prefetch_multiplier: int | None = None,
        disable_rate_limits: bool | None = None,
        worker_lost_wait: float | None = None,
        max_memory_per_child: int | None = None,
        **_kw: Any,
    ) -> dict[str, Any]: ...
    def setup_includes(self, includes: Sequence[str]) -> None: ...
    def setup_instance(
        self,
        queues: Sequence[str] | None = None,
        ready_callback: Callable[[], None] | None = None,
        pidfile: str | None = None,
        include: Sequence[str] | None = None,
        use_eventloop: bool | None = None,
        exclude_queues: Sequence[str] | None = None,
        **kwargs: Any,
    ) -> None: ...
    def setup_queues(
        self, include: Sequence[str], exclude: Sequence[str] | None = None
    ) -> None: ...
    def should_use_eventloop(self) -> bool: ...
    def signal_consumer_close(self) -> None: ...
    def start(self) -> None: ...
    def stats(self) -> dict[str, Any]: ...
    def stop(
        self, in_sighandler: bool = False, exitcode: int | None = None
    ) -> None: ...
    def terminate(self, in_sighandler: bool = False) -> None: ...
    def wait_for_soft_shutdown(self) -> bool: ...



from collections.abc import Callable, Iterable, Mapping
from datetime import datetime
from typing import (
    Any,
    Generic,
    overload,
)

__all__ = (
    "Signature",
    "chain",
    "chord",
    "chunks",
    "group",
    "maybe_signature",
    "signature",
    "xmap",
    "xstarmap",
)

import celery.result
import kombu
from celery.app.base import Celery
from celery.app.task import Task
from celery.result import EagerResult
from celery.utils import abstract
from typing_extensions import TypeVar, override

_F = TypeVar("_F", bound=Callable[..., Any])
_R_co = TypeVar("_R_co", covariant=True, default=Any)

class Signature(dict[str, Any], Generic[_R_co]):
    TYPES: dict[str, type[Signature[Any]]]  # ty: ignore[invalid-type-form]

    @classmethod
    def register_type(
        cls, name: str | None = None
    ) -> Callable[[type[Signature[Any]]], type[Signature[Any]]]: ...  # ty: ignore[invalid-type-form]
    @classmethod
    def from_dict(
        cls, d: dict[str, Any], app: Celery | None = ...
    ) -> Signature[Any]: ...
    def __init__(
        self,
        task: Task[Any, _R_co] | str | None = ...,
        args: tuple[Any, ...] | None = ...,
        kwargs: dict[str, Any] | None = ...,
        options: dict[str, Any] | None = ...,
        type: Any | None = ...,
        subtask_type: Any | None = ...,
        immutable: bool = ...,
        app: Celery | None = ...,
        *,
        # **ex expanded
        task_id: str | None = ...,
        producer: kombu.Producer | None = ...,
        link: Signature[Any] | None = ...,
        link_error: Signature[Any] | None = ...,
        shadow: str | None = ...,
        # apply_async options
        countdown: float = ...,
        eta: datetime | None = ...,
        expires: float | datetime = ...,
        retry: bool = ...,
        retry_policy: Mapping[str, Any] = ...,
        queue: str | kombu.Queue = ...,
        exchange: str | kombu.Exchange = ...,
        routing_key: str = ...,
        priority: int = ...,
        serializer: str = ...,
        compression: str = ...,
        add_to_parent: bool = ...,
        publisher: kombu.Producer = ...,
        headers: dict[str, str] = ...,
    ) -> None: ...
    def __call__(self, *partial_args: Any, **partial_kwargs: Any) -> _R_co: ...
    def delay(
        self, *partial_args: Any, **partial_kwargs: Any
    ) -> celery.result.AsyncResult[_R_co]: ...
    def apply(
        self,
        args: tuple[Any, ...] | None = ...,
        kwargs: dict[str, Any] | None = ...,
        **options: Any,
    ) -> EagerResult[_R_co]: ...
    def apply_async(
        self,
        args: tuple[Any, ...] | None = ...,
        kwargs: dict[str, Any] | None = ...,
        route_name: str | None = ...,
        *,
        # options (captured by **options at runtime)
        task_id: str | None = ...,
        producer: kombu.Producer | None = ...,
        link: Signature[Any] | list[Signature[Any]] | None = ...,
        link_error: Signature[Any] | list[Signature[Any]] | None = ...,
        shadow: str | None = ...,
        # apply_async options
        countdown: float = ...,
        eta: datetime | None = ...,
        expires: float | datetime = ...,
        retry: bool = ...,
        retry_policy: Mapping[str, Any] = ...,
        queue: str | kombu.Queue = ...,
        exchange: str | kombu.Exchange = ...,
        routing_key: str = ...,
        priority: int = ...,
        serializer: str = ...,
        compression: str = ...,
        add_to_parent: bool = ...,
        publisher: kombu.Producer = ...,
        headers: dict[str, str] = ...,
        **options: Any,
    ) -> celery.result.AsyncResult[_R_co]: ...
    def clone(
        self,
        args: tuple[Any, ...] | None = ...,
        kwargs: dict[str, Any] | None = ...,
        *,
        # **ex expanded
        task_id: str | None = ...,
        producer: kombu.Producer | None = ...,
        link: Signature[Any] | None = ...,
        link_error: Signature[Any] | None = ...,
        shadow: str | None = ...,
        # apply_async options
        countdown: float = ...,
        eta: datetime | None = ...,
        expires: float | datetime = ...,
        retry: bool = ...,
        retry_policy: Mapping[str, Any] = ...,
        queue: str | kombu.Queue = ...,
        exchange: str | kombu.Exchange = ...,
        routing_key: str = ...,
        priority: int = ...,
        serializer: str = ...,
        compression: str = ...,
        add_to_parent: bool = ...,
        publisher: kombu.Producer = ...,
        headers: dict[str, str] = ...,
    ) -> Signature[_R_co]: ...
    partial = clone
    def freeze(
        self,
        _id: str | None = ...,
        group_id: str | None = ...,
        chord: chord | None = ...,
        root_id: str | None = ...,
        parent_id: str | None = ...,
        group_index: int | None = ...,
    ) -> celery.result.AsyncResult[_R_co]: ...
    def replace(
        self,
        args: tuple[Any, ...] | None = ...,
        kwargs: dict[str, Any] | None = ...,
        options: dict[str, Any] | None = ...,
    ) -> Signature[_R_co]: ...
    def set(
        self,
        immutable: bool | None = ...,
        *,
        # **options expanded
        task_id: str | None = ...,
        producer: kombu.Producer | None = ...,
        link: Signature[Any] | None = ...,
        link_error: Signature[Any] | None = ...,
        shadow: str | None = ...,
        # apply_async options
        countdown: float = ...,
        eta: datetime | None = ...,
        expires: float | datetime = ...,
        retry: bool = ...,
        retry_policy: Mapping[str, Any] = ...,
        queue: str | kombu.Queue = ...,
        exchange: str | kombu.Exchange = ...,
        routing_key: str = ...,
        priority: int = ...,
        serializer: str = ...,
        compression: str = ...,
        add_to_parent: bool = ...,
        publisher: kombu.Producer = ...,
        headers: dict[str, str] = ...,
    ) -> Signature[_R_co]: ...
    def set_immutable(self, immutable: bool) -> None: ...
    def append_to_list_option(self, key: str, value: Any) -> Any: ...
    def extend_list_option(self, key: str, value: Any) -> None: ...
    def link(self, callback: _F) -> _F: ...
    def link_error(self, errback: Callable[..., Any]) -> Signature[_R_co]: ...
    def on_error(self, errback: _F) -> _F: ...
    def flatten_links(self) -> list[Signature[Any]]: ...
    def stamp(
        self,
        visitor: Any = ...,
        append_stamps: bool = ...,
        **headers: Any,
    ) -> Any: ...
    def stamp_links(
        self,
        visitor: Any,
        append_stamps: bool = ...,
        **headers: Any,
    ) -> Any: ...
    def __deepcopy__(self, memo: dict[int, Any]) -> Signature[_R_co]: ...
    def __invert__(self) -> Any: ...
    def __json__(self) -> dict[str, Any]: ...
    def reprcall(self, *args: Any, **kwargs: Any) -> str: ...
    # TODO(sbdchd): use overloads to properly type this
    @override
    def __or__(  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
        self, other: Signature[Any]
    ) -> Signature[Any]: ...
    def election(self) -> celery.result.AsyncResult[_R_co]: ...
    @property
    def name(self) -> str: ...
    @property
    def type(self) -> Any: ...
    @property
    def app(self) -> Celery: ...
    @property
    def AsyncResult(self) -> celery.result.AsyncResult[_R_co]: ...
    @property
    def id(self) -> str | None: ...
    @property
    def parent_id(self) -> str | None: ...
    @property
    def root_id(self) -> str | None: ...
    @property
    def task(self) -> str | None: ...
    @property
    def args(self) -> tuple[Any, ...]: ...
    @property
    def kwargs(self) -> dict[str, Any]: ...
    @property
    def options(self) -> dict[str, Any]: ...
    @property
    def subtask_type(self) -> Any: ...
    @property
    def immutable(self) -> bool: ...

class _chain(Signature[Any]):
    @property
    def tasks(self) -> tuple[Signature[Any], ...]: ...
    def __init__(
        self,
        *tasks: Signature[Any],
        # Signature extras
        options: dict[str, Any] | None = ...,
        type: Any | None = ...,
        subtask_type: Any | None = ...,
        immutable: bool = ...,
        app: Celery | None = ...,
        task_id: str | None = ...,
        producer: kombu.Producer | None = ...,
        link: Signature[Any] | None = ...,
        link_error: Signature[Any] | None = ...,
        shadow: str | None = ...,
        countdown: float = ...,
        eta: datetime | None = ...,
        expires: float | datetime = ...,
        retry: bool = ...,
        retry_policy: Mapping[str, Any] = ...,
        queue: str | kombu.Queue = ...,
        exchange: str | kombu.Exchange = ...,
        routing_key: str = ...,
        priority: int = ...,
        serializer: str = ...,
        compression: str = ...,
        add_to_parent: bool = ...,
        publisher: kombu.Producer = ...,
        headers: dict[str, str] = ...,
    ) -> None: ...
    @override
    def apply_async(  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
        self,
        args: tuple[Any, ...] | None = ...,
        kwargs: dict[str, Any] | None = ...,
        **options: Any,
    ) -> celery.result.AsyncResult[Any]: ...
    def prepare_steps(
        self,
        args: tuple[Any, ...],
        kwargs: dict[str, Any],
        tasks: list[Signature[Any]],
        root_id: str | None = None,
        parent_id: str | None = None,
        link_error: Signature[Any] | None = None,
        app: Celery | None = None,
        last_task_id: str | None = None,
        group_id: str | None = None,
        chord_body: Signature[Any] | None = None,
        clone: bool = True,
        from_dict: Callable[[dict[str, Any], Celery | None], Signature[Any]] = ...,
        group_index: int | None = None,
    ) -> tuple[Any, ...]: ...
    def run(
        self,
        args: tuple[Any, ...] | None = None,
        kwargs: dict[str, Any] | None = None,
        group_id: str | None = None,
        chord: chord | None = None,
        task_id: str | None = None,
        link: Signature[Any] | None = None,
        link_error: Signature[Any] | None = None,
        publisher: kombu.Producer | None = None,
        producer: kombu.Producer | None = None,
        root_id: str | None = None,
        parent_id: str | None = None,
        app: Celery | None = None,
        group_index: int | None = None,
        **options: Any,
    ) -> celery.result.AsyncResult[Any]: ...
    def unchain_tasks(self) -> list[Signature[Any]]: ...

class chain(_chain): ...

class _basemap(Signature[Any]):
    def __init__(
        self,
        task: Task[Any, Any] | None,
        it: Iterable[Any],
        *,
        # Signature extras
        args: tuple[Any, ...] | None = ...,
        kwargs: dict[str, Any] | None = ...,
        options: dict[str, Any] | None = ...,
        type: Any | None = ...,
        subtask_type: Any | None = ...,
        immutable: bool = ...,
        app: Celery | None = ...,
        task_id: str | None = ...,
        producer: kombu.Producer | None = ...,
        link: Signature[Any] | None = ...,
        link_error: Signature[Any] | None = ...,
        shadow: str | None = ...,
        countdown: float = ...,
        eta: datetime | None = ...,
        expires: float | datetime = ...,
        retry: bool = ...,
        retry_policy: Mapping[str, Any] = ...,
        queue: str | kombu.Queue = ...,
        exchange: str | kombu.Exchange = ...,
        routing_key: str = ...,
        priority: int = ...,
        serializer: str = ...,
        compression: str = ...,
        add_to_parent: bool = ...,
        publisher: kombu.Producer = ...,
        headers: dict[str, str] = ...,
    ) -> None: ...
    @override
    def apply_async(  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
        self,
        args: tuple[Any, ...] | None = ...,
        kwargs: dict[str, Any] | None = ...,
        **opts: Any,
    ) -> celery.result.AsyncResult[Any]: ...

class xmap(_basemap): ...
class xstarmap(_basemap): ...

class chunks(Signature[Any]):
    def __init__(
        self,
        task: Task[Any, Any] | None,
        it: Iterable[Any],
        n: int,
        *,
        # Signature extras
        args: tuple[Any, ...] | None = ...,
        kwargs: dict[str, Any] | None = ...,
        options: dict[str, Any] | None = ...,
        type: Any | None = ...,
        subtask_type: Any | None = ...,
        immutable: bool = ...,
        app: Celery | None = ...,
        task_id: str | None = ...,
        producer: kombu.Producer | None = ...,
        link: Signature[Any] | None = ...,
        link_error: Signature[Any] | None = ...,
        shadow: str | None = ...,
        countdown: float = ...,
        eta: datetime | None = ...,
        expires: float | datetime = ...,
        retry: bool = ...,
        retry_policy: Mapping[str, Any] = ...,
        queue: str | kombu.Queue = ...,
        exchange: str | kombu.Exchange = ...,
        routing_key: str = ...,
        priority: int = ...,
        serializer: str = ...,
        compression: str = ...,
        add_to_parent: bool = ...,
        publisher: kombu.Producer = ...,
        headers: dict[str, str] = ...,
    ) -> None: ...
    @classmethod
    def apply_chunks(
        cls,
        task: Task[Any, Any],
        it: Iterable[Any],
        n: int,
        app: Celery | None = ...,
    ) -> list[Any]: ...
    @override
    def __call__(self, **options: Any) -> Any: ...
    @override
    def apply_async(  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
        self,
        args: tuple[Any, ...] | None = ...,
        kwargs: dict[str, Any] | None = ...,
        **opts: Any,
    ) -> celery.result.AsyncResult[Any]: ...
    def group(self) -> _group: ...

class group(Signature[Any]):
    @property
    def tasks(self) -> list[Signature[Any]]: ...
    @overload
    def __init__(
        self,
        *tasks: group | abstract.CallableSignature | Iterable[Signature[Any]],
        **options: Any,
    ) -> None: ...
    @overload
    def __init__(
        self,
        *tasks: Signature[Any],
        # Signature extras
        args: tuple[Any, ...] | None = ...,
        kwargs: dict[str, Any] | None = ...,
        options: dict[str, Any] | None = ...,
        type: Any | None = ...,
        subtask_type: Any | None = ...,
        immutable: bool = ...,
        app: Celery | None = ...,
        task_id: str | None = ...,
        producer: kombu.Producer | None = ...,
        link: Signature[Any] | None = ...,
        link_error: Signature[Any] | None = ...,
        shadow: str | None = ...,
        countdown: float = ...,
        eta: datetime | None = ...,
        expires: float | datetime = ...,
        retry: bool = ...,
        retry_policy: Mapping[str, Any] = ...,
        queue: str | kombu.Queue = ...,
        exchange: str | kombu.Exchange = ...,
        routing_key: str = ...,
        priority: int = ...,
        serializer: str = ...,
        compression: str = ...,
        add_to_parent: bool = ...,
        publisher: kombu.Producer = ...,
        headers: dict[str, str] = ...,
    ) -> None: ...
    def skew(
        self, start: float = ..., stop: float | None = ..., step: float = ...
    ) -> group: ...
    @override
    def apply_async(  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
        self,
        args: tuple[Any, ...] | None = ...,
        kwargs: dict[str, Any] | None = ...,
        add_to_parent: bool = ...,
        producer: kombu.Producer | None = ...,
        link: Signature[Any] | list[Signature[Any]] | None = ...,
        link_error: Signature[Any] | list[Signature[Any]] | None = ...,
        **options: Any,
    ) -> celery.result.AsyncResult[Any]: ...
    @override
    def link(self, sig: Signature[Any]) -> Signature[Any]: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
    @override
    def link_error(self, sig: Signature[Any]) -> Signature[Any]: ...  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
    @override
    def stamp(
        self,
        visitor: Any = ...,
        append_stamps: bool = ...,
        **headers: Any,
    ) -> Any: ...
    @override
    def __or__(self, other: Signature[Any]) -> chord: ...  # type: ignore[override]

_group = group

class chord(Signature[Any]):
    @property
    def tasks(self) -> Any: ...
    @property
    def body(self) -> Any: ...
    def __init__(
        self,
        header: Any,
        body: Any | None = ...,
        task: str = ...,
        args: tuple[Any, ...] | None = ...,
        kwargs: dict[str, Any] | None = ...,
        app: Celery | None = ...,
        *,
        # from Signature
        options: dict[str, Any] | None = ...,
        type: Any | None = ...,
        subtask_type: Any | None = ...,
        immutable: bool = ...,
        task_id: str | None = ...,
        producer: kombu.Producer | None = ...,
        link: Signature[Any] | None = ...,
        link_error: Signature[Any] | None = ...,
        shadow: str | None = ...,
        countdown: float = ...,
        eta: datetime | None = ...,
        expires: float | datetime = ...,
        retry: bool = ...,
        retry_policy: Mapping[str, Any] = ...,
        queue: str | kombu.Queue = ...,
        exchange: str | kombu.Exchange = ...,
        routing_key: str = ...,
        priority: int = ...,
        serializer: str = ...,
        compression: str = ...,
        add_to_parent: bool = ...,
        publisher: kombu.Producer = ...,
        headers: dict[str, str] = ...,
    ) -> None: ...
    def run(
        self,
        header: Any,
        body: Any,
        partial_args: tuple[Any, ...],
        app: Celery | None = ...,
        interval: float | None = ...,
        countdown: int = ...,
        max_retries: int | None = ...,
        eager: bool = ...,
        task_id: str | None = ...,
        kwargs: dict[str, Any] | None = ...,
        **options: Any,
    ) -> Any: ...
    def __length_hint__(self) -> int: ...
    @override
    def apply(
        self,
        args: tuple[Any, ...] | None = ...,
        kwargs: dict[str, Any] | None = ...,
        propagate: bool = ...,
        body: Signature[Any] | None = ...,
        **options: Any,
    ) -> EagerResult[Any]: ...
    @override
    def apply_async(  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
        self,
        args: tuple[Any, ...] | None = ...,
        kwargs: dict[str, Any] | None = ...,
        task_id: str | None = ...,
        producer: kombu.Producer | None = ...,
        publisher: kombu.Producer | None = ...,
        connection: kombu.Connection | None = ...,
        router: Any | None = ...,
        result_cls: type[celery.result.AsyncResult[Any]] | None = ...,
        **options: Any,
    ) -> celery.result.AsyncResult[Any]: ...
    @override
    def stamp(
        self,
        visitor: Any = ...,
        append_stamps: bool = ...,
        **headers: Any,
    ) -> Any: ...
    @override
    def __or__(self, other: Signature[Any]) -> chord: ...  # type: ignore[override]
    @override
    def __call__(
        self,
        body: Signature[Any] | None = ...,
        **options: Any,
    ) -> celery.result.AsyncResult[Any]: ...

def signature(
    varies: Signature[Any] | str | dict[str, Any], *args: Any, **kwargs: Any
) -> Signature[Any]: ...

subtask = signature

def maybe_signature(
    d: abstract.CallableSignature | Mapping[str, Any] | None,
    app: Celery | None = ...,
    clone: bool = ...,
) -> abstract.CallableSignature | None: ...

maybe_subtask = maybe_signature



from collections.abc import Callable
from datetime import timedelta
from typing import Any, NamedTuple

__all__ = ("BaseBackend", "DisabledBackend", "KeyValueStoreBackend")

from celery.app.task import Context
from celery.exceptions import TimeoutError as CeleryTimeoutError
from celery.result import ResultSet
from typing_extensions import override

class pending_results_t(NamedTuple):
    concrete: dict[Any, Any]
    weak: dict[Any, Any]

class Backend:
    READY_STATES: frozenset[str]
    UNREADY_STATES: frozenset[str]
    EXCEPTION_STATES: frozenset[str]

    TimeoutError: type[CeleryTimeoutError]

    subpolling_interval: float | None
    supports_native_join: bool
    supports_autoexpire: bool
    persistent: bool

    retry_policy: dict[str, int | float]
    def __init__(
        self,
        app: Any,
        serializer: str | None = ...,
        max_cached_results: int | None = ...,
        accept: list[str] | None = ...,
        expires: float | timedelta | None = ...,
        expires_type: Callable[[Any], Any] | None = ...,
        url: str | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def as_uri(self, include_password: bool = ...) -> str: ...
    def mark_as_started(self, task_id: str, **meta: Any) -> None: ...
    def mark_as_done(
        self,
        task_id: str,
        result: Any,
        request: Any = ...,
        store_result: bool = ...,
        state: str = ...,
    ) -> None: ...
    def mark_as_failure(
        self,
        task_id: str,
        exc: Exception,
        traceback: str | None = ...,
        request: Any = ...,
        store_result: bool = ...,
        call_errbacks: bool = ...,
        state: str = ...,
    ) -> None: ...
    def mark_as_revoked(
        self,
        task_id: str,
        reason: str = ...,
        request: Any = ...,
        store_result: bool = ...,
        state: str = ...,
    ) -> None: ...
    def mark_as_retry(
        self,
        task_id: str,
        exc: Exception,
        traceback: str | None = ...,
        request: Any = ...,
        store_result: bool = ...,
        state: str = ...,
    ) -> None: ...
    def store_result(
        self,
        task_id: str,
        result: Any,
        state: str,
        traceback: str | None = ...,
        request: Any = ...,
        **kwargs: Any,
    ) -> Any: ...
    def forget(self, task_id: str) -> None: ...
    def get_state(self, task_id: str) -> str: ...
    def get_result(self, task_id: str) -> Any: ...
    def get_children(self, task_id: str) -> list[Any] | None: ...
    def reload_task_result(self, task_id: str) -> None: ...
    def reload_group_result(self, group_id: str) -> None: ...
    def get_group_meta(self, group_id: str, cache: bool = ...) -> Any: ...
    def restore_group(self, group_id: str, cache: bool = ...) -> ResultSet | None: ...
    def cleanup(self) -> None: ...
    def process_cleanup(self) -> None: ...
    def apply_chord(
        self, header_result_args: tuple[Any, Any], body: Any, **kwargs: Any
    ) -> None: ...
    def current_task_children(self, request: Context | None = ...) -> list[Any]: ...
    def add_to_chord(self, chord_id: str, result: Any) -> None: ...
    def chord_error_from_stack(
        self, callback: Any, exc: Exception | None = None
    ) -> Exception: ...
    def decode(self, payload: bytes) -> Any: ...
    def decode_result(self, payload: bytes) -> Any: ...
    def delete_group(self, group_id: str) -> None: ...
    def encode(self, data: Any) -> bytes: ...
    def encode_result(self, result: Any, state: str) -> bytes: ...
    def ensure_chords_allowed(self) -> None: ...
    def exception_safe_to_retry(self, exc: Exception) -> bool: ...
    def exception_to_python(self, exc: Any) -> Exception: ...
    def fail_from_current_stack(
        self, task_id: str, exc: Exception | None = None
    ) -> None: ...
    def fallback_chord_unlock(
        self, header_result: Any, body: Any, countdown: int = 1, **kwargs: Any
    ) -> None: ...
    def get_status(self, task_id: str) -> str: ...
    def get_task_meta(self, task_id: str, cache: bool = True) -> dict[str, Any]: ...
    def get_traceback(self, task_id: str) -> str | None: ...
    def is_cached(self, task_id: str) -> bool: ...
    def meta_from_decoded(self, meta: dict[str, Any]) -> dict[str, Any]: ...
    def on_chord_part_return(
        self, request: Any, state: str, result: Any, **kwargs: Any
    ) -> None: ...
    def on_task_call(self, producer: Any, task_id: str) -> None: ...
    def prepare_exception(
        self, exc: Exception, serializer: str | None = None
    ) -> Any: ...
    def prepare_expires(
        self, value: float | timedelta | None, type: type | None = None
    ) -> float | None: ...
    def prepare_persistent(self, enabled: bool | None = None) -> bool: ...
    def prepare_value(self, result: Any) -> Any: ...
    def save_group(self, group_id: str, result: Any) -> None: ...
    def set_chord_size(self, group_id: str, chord_size: int) -> None: ...

class SyncBackendMixin:
    @property
    def is_async(self) -> bool: ...
    def add_pending_result(self, result: Any, weak: bool = False) -> Any: ...
    def remove_pending_result(self, result: Any) -> None: ...
    def iter_native(
        self,
        result: ResultSet,
        timeout: float | None = ...,
        interval: float = ...,
        no_ack: bool = ...,
        on_message: Callable[[Any], None] | None = ...,
        on_interval: Callable[[], None] | None = ...,
    ) -> None: ...
    def wait_for_pending(
        self,
        result: ResultSet,
        timeout: float | None = ...,
        interval: float = ...,
        no_ack: bool = ...,
        on_message: Callable[[Any], None] | None = ...,
        on_interval: Callable[[], None] | None = ...,
        callback: Callable[[Any], Any] | None = ...,
        propagate: bool = ...,
    ) -> None: ...
    def wait_for(
        self,
        task_id: str,
        timeout: float | None = ...,
        interval: float = ...,
        no_ack: bool = ...,
        on_interval: Callable[[], None] | None = ...,
    ) -> Any: ...

class BaseBackend(Backend, SyncBackendMixin): ...

class BaseKeyValueStoreBackend(Backend):
    chord_keyprefix: str
    group_keyprefix: str
    task_keyprefix: str
    implements_incr: bool
    key_t: Callable[[str], bytes]
    def get(self, key: str) -> Any: ...
    def mget(self, keys: list[str]) -> list[Any]: ...
    def set(self, key: str, value: Any) -> None: ...
    def delete(self, key: str) -> None: ...
    def incr(self, key: str) -> int: ...
    def expire(self, key: str, value: int) -> None: ...
    def get_key_for_chord(self, group_id: str, key: str = "") -> str: ...
    def get_key_for_group(self, group_id: str, key: str = "") -> str: ...
    def get_key_for_task(self, task_id: str, key: str = "") -> str: ...
    def get_many(
        self,
        task_ids: list[str],
        timeout: float | None = None,
        interval: float = 0.5,
        no_ack: bool = True,
        on_message: Callable[[Any], None] | None = None,
        on_interval: Callable[[], None] | None = None,
        max_iterations: int | None = None,
        READY_STATES: frozenset[str] = ...,
    ) -> Any: ...
    @override
    def on_chord_part_return(
        self, request: Any, state: str, result: Any, **kwargs: Any
    ) -> None: ...

class KeyValueStoreBackend(BaseKeyValueStoreBackend, SyncBackendMixin): ...

class DisabledBackend(BaseBackend):
    @override
    def store_result(self, *args: Any, **kwargs: Any) -> None: ...
    @override
    def ensure_chords_allowed(self) -> None: ...
    def get_many(self, *args: Any, **kwargs: Any) -> Any: ...
    @override
    def get_status(self, *args: Any, **kwargs: Any) -> str: ...
    def get_task_meta_for(self, *args: Any, **kwargs: Any) -> Any: ...
    @override
    def get_traceback(self, *args: Any, **kwargs: Any) -> str | None: ...



from collections.abc import Callable
from datetime import datetime, timedelta, tzinfo
from time import struct_time
from types import ModuleType
from typing import Any

from typing_extensions import override

__all__ = (
    "LocalTimezone",
    "adjust_timestamp",
    "delta_resolution",
    "ffwd",
    "get_exponential_backoff_interval",
    "humanize_seconds",
    "is_naive",
    "localize",
    "make_aware",
    "maybe_iso8601",
    "maybe_make_aware",
    "maybe_timedelta",
    "rate",
    "remaining",
    "timezone",
    "to_utc",
    "utcoffset",
    "weekday",
)

class LocalTimezone(tzinfo):
    @override
    def utcoffset(self, dt: datetime | None) -> timedelta: ...
    @override
    def dst(self, dt: datetime | None) -> timedelta: ...
    @override
    def tzname(self, dt: datetime | None) -> str: ...
    @override
    def fromutc(self, dt: datetime) -> datetime: ...

class _Zone:
    @property
    def local(self) -> tzinfo: ...
    @property
    def utc(self) -> tzinfo: ...
    def get_timezone(self, zone: str | tzinfo) -> tzinfo: ...
    def to_local(
        self,
        dt: datetime,
        local: tzinfo | None = None,
        orig: tzinfo | None = None,
    ) -> datetime: ...
    def to_local_fallback(self, dt: datetime) -> datetime: ...
    def to_system(self, dt: datetime) -> datetime: ...
    def tz_or_local(self, tzinfo: tzinfo | None = None) -> tzinfo: ...

timezone: _Zone

class ffwd:
    def __init__(
        self,
        year: int | None = None,
        month: int | None = None,
        weeks: int = 0,
        weekday: int | None = None,
        day: int | None = None,
        hour: int | None = None,
        minute: int | None = None,
        second: int | None = None,
        microsecond: int | None = None,
        **kwargs: Any,
    ) -> None: ...
    def __radd__(self, other: Any) -> timedelta: ...

def adjust_timestamp(
    ts: float,
    offset: int,
    here: Callable[..., float] = ...,
) -> float: ...
def delta_resolution(dt: datetime, delta: timedelta) -> datetime: ...
def get_exponential_backoff_interval(
    factor: int,
    retries: int,
    maximum: int,
    full_jitter: bool = False,
) -> int: ...
def humanize_seconds(
    secs: int,
    prefix: str = "",
    sep: str = "",
    now: str = "now",
    microseconds: bool = False,
) -> str: ...
def is_naive(dt: datetime) -> bool: ...
def localize(dt: datetime, tz: tzinfo) -> datetime: ...
def make_aware(dt: datetime, tz: tzinfo) -> datetime: ...
def maybe_iso8601(dt: datetime | str | None) -> datetime | None: ...
def maybe_make_aware(
    dt: datetime,
    tz: tzinfo | None = None,
    naive_as_utc: bool = True,
) -> datetime: ...
def maybe_timedelta(delta: int) -> timedelta: ...
def rate(r: str) -> float: ...
def remaining(
    start: datetime,
    ends_in: timedelta,
    now: datetime | None = None,
    relative: bool = False,
) -> timedelta: ...
def to_utc(dt: datetime) -> datetime: ...
def utcoffset(
    time: ModuleType = ...,
    localtime: Callable[..., struct_time] = ...,
) -> float: ...
def weekday(name: str) -> int: ...



import logging
import types
from collections import OrderedDict
from collections.abc import Callable, Generator, Iterable
from typing import Any, NamedTuple, TypeAlias

from amqp.protocol import queue_declare_ok_t
from kombu.connection import Connection
from kombu.message import Message as BaseMessage
from kombu.transport.base import Management as BaseManagement
from kombu.transport.base import StdChannel
from kombu.transport.base import Transport as BaseTransport
from kombu.transport.virtual.exchange import ExchangeType
from kombu.utils.scheduling import FairCycle
from typing_extensions import Self, override

ARRAY_TYPE_H: str
NOT_EQUIVALENT_FMT: str
RESTORE_PANIC_FMT: str
RESTORING_FMT: str
UNDELIVERABLE_FMT: str
W_NO_CONSUMERS: str

logger: logging.Logger

# Forward references for type annotations to avoid name collisions with class attributes
_QoSType: TypeAlias = QoS
_MessageType: TypeAlias = Message
_ChannelType: TypeAlias = Channel

class binding_key_t(NamedTuple):
    queue: str
    exchange: str
    routing_key: str

class queue_binding_t(NamedTuple):
    exchange: str
    routing_key: str
    arguments: dict[str, Any] | None

class Base64:
    def encode(self, s: str | bytes) -> str: ...
    def decode(self, s: str) -> bytes: ...

class NotEquivalentError(Exception): ...
class UndeliverableWarning(UserWarning): ...

class BrokerState:
    exchanges: dict[str, dict[str, Any]] | None
    bindings: dict[binding_key_t, queue_binding_t] | None
    queue_index: dict[str, set[binding_key_t]] | None

    def __init__(self, exchanges: dict[str, dict[str, Any]] | None = ...) -> None: ...
    def clear(self) -> None: ...
    def has_binding(self, queue: str, exchange: str, routing_key: str) -> bool: ...
    def binding_declare(
        self,
        queue: str,
        exchange: str,
        routing_key: str,
        arguments: dict[str, Any] | None,
    ) -> None: ...
    def binding_delete(self, queue: str, exchange: str, routing_key: str) -> None: ...
    def queue_bindings_delete(self, queue: str) -> None: ...
    def queue_bindings(self, queue: str) -> Generator[queue_binding_t, None, None]: ...

class QoS:
    channel: Channel
    prefetch_count: int
    restore_at_shutdown: bool

    _delivered: OrderedDict[str, Any] | None
    _dirty: set[str] | None

    def __init__(self, channel: Channel, prefetch_count: int = ...) -> None: ...
    def can_consume(self) -> bool: ...
    def can_consume_max_estimate(self) -> int: ...
    def append(self, message: Any, delivery_tag: str) -> None: ...
    def get(self, delivery_tag: str) -> Any: ...
    def ack(self, delivery_tag: str) -> None: ...
    def reject(self, delivery_tag: str, requeue: bool = ...) -> None: ...
    def restore_unacked(self) -> None: ...
    def restore_unacked_once(self, stderr: Any | None = ...) -> None: ...
    def restore_visible(self, *args: Any, **kwargs: Any) -> None: ...

class Message(BaseMessage):
    def __init__(
        self,
        payload: dict[str, Any],
        channel: Channel | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def serializable(self) -> dict[str, Any]: ...

# AbstractChannel is a base class with only internal methods
class AbstractChannel:
    def _get(self, queue: str, timeout: float | None = ...) -> Any: ...
    def _put(self, queue: str, message: Any) -> None: ...
    def _purge(self, queue: str) -> int: ...
    def _size(self, queue: str) -> int: ...
    def _delete(self, queue: str, *args: Any, **kwargs: Any) -> None: ...
    def _new_queue(self, queue: str, **kwargs: Any) -> None: ...
    def _has_queue(self, queue: str, **kwargs: Any) -> bool: ...
    def _poll(
        self,
        cycle: FairCycle,
        callback: Callable[..., Any],
        timeout: float | None = ...,
    ) -> None: ...

# Channel inherits from AbstractChannel and StdChannel (multiple inheritance)
class Channel(AbstractChannel, StdChannel):
    Message: type[Message]
    QoS: type[QoS]

    do_restore: bool
    exchange_types: dict[str, type[ExchangeType]]
    supports_fanout: bool

    codecs: dict[str, Base64]
    body_encoding: str

    default_priority: int
    min_priority: int
    max_priority: int
    deadletter_queue: str | None

    connection: Transport

    closed: bool
    _active_queues: list[str]
    _delivery_tags: Iterable[int]
    _consumers: set[str]
    _cycle: FairCycle | None
    _qos: _QoSType | None
    _tag_to_queue: dict[str, str]

    from_transport_options: tuple[str, ...]

    # NOTE: Despite the name, 'connection' is actually a Transport (from establish_connection())
    def __init__(self, connection: Transport, **kwargs: Any) -> None: ...
    def exchange_declare(
        self,
        exchange: str | None = ...,
        type: str = ...,
        durable: bool = ...,
        auto_delete: bool = ...,
        arguments: dict[str, Any] | None = ...,
        nowait: bool = ...,
        passive: bool = ...,
    ) -> None: ...
    def exchange_delete(
        self,
        exchange: str,
        if_unused: bool = ...,
        nowait: bool = ...,
    ) -> None: ...
    def exchange_bind(
        self,
        destination: str,
        source: str = ...,
        routing_key: str = ...,
        nowait: bool = ...,
        arguments: dict[str, Any] | None = ...,
    ) -> None: ...
    def exchange_unbind(
        self,
        destination: str,
        source: str = ...,
        routing_key: str = ...,
        nowait: bool = ...,
        arguments: dict[str, Any] | None = ...,
    ) -> None: ...
    def queue_declare(
        self,
        queue: str | None = ...,
        passive: bool = ...,
        **kwargs: Any,
    ) -> queue_declare_ok_t: ...
    def queue_delete(
        self,
        queue: str,
        if_unused: bool = ...,
        if_empty: bool = ...,
        **kwargs: Any,
    ) -> int | None: ...
    def queue_bind(
        self,
        queue: str,
        exchange: str | None = ...,
        routing_key: str = ...,
        arguments: dict[str, Any] | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def queue_unbind(
        self,
        queue: str,
        exchange: str | None = ...,
        routing_key: str = ...,
        arguments: dict[str, Any] | None = ...,
        **kwargs: Any,
    ) -> None: ...
    def queue_purge(self, queue: str, **kwargs: Any) -> int | None: ...
    def basic_publish(
        self,
        message: Any,
        exchange: str,
        routing_key: str,
        **kwargs: Any,
    ) -> None: ...
    def basic_consume(
        self,
        queue: str,
        no_ack: bool,
        callback: Callable[..., Any] | None,
        consumer_tag: str | None,
        **kwargs: Any,
    ) -> str: ...
    def basic_cancel(self, consumer_tag: str) -> Any: ...
    def basic_get(
        self,
        queue: str,
        no_ack: bool = ...,
        **kwargs: Any,
    ) -> _MessageType | None: ...
    def basic_ack(self, delivery_tag: str, multiple: bool = ...) -> None: ...
    def basic_qos(
        self,
        prefetch_size: int = ...,
        prefetch_count: int = ...,
        apply_global: bool = ...,
    ) -> None: ...
    def basic_recover(self, requeue: bool = ...) -> None: ...
    def basic_reject(self, delivery_tag: str, requeue: bool = ...) -> None: ...
    def close(self) -> None: ...
    def encode_body(
        self, body: bytes, encoding: str | None = ...
    ) -> tuple[Any, str]: ...
    def decode_body(self, body: Any, encoding: str | None = ...) -> bytes: ...
    def drain_events(
        self, timeout: float | None = ..., callback: Callable[..., Any] | None = ...
    ) -> None: ...
    def get_exchanges(self) -> list[str]: ...
    def get_table(self, exchange: str) -> list[tuple[str, str, str]]: ...
    def typeof(self, exchange: str, default: str = ...) -> ExchangeType: ...
    def list_bindings(self) -> Generator[tuple[str, str, str], None, None]: ...
    def message_to_python(self, raw_message: Any) -> _MessageType: ...
    def prepare_message(
        self,
        body: Any,
        priority: int | None = ...,
        content_type: str | None = ...,
        content_encoding: str | None = ...,
        headers: dict[str, Any] | None = ...,
        properties: dict[str, Any] | None = ...,
    ) -> dict[str, Any]: ...
    def flow(self, active: bool = ...) -> None: ...
    def _next_delivery_tag(self) -> str: ...
    def _get_message_priority(self, message: Any, reverse: bool = ...) -> int: ...
    def _restore(self, message: Any) -> None: ...
    def _restore_at_beginning(self, message: Any) -> None: ...
    def _get_and_deliver(self, queue: str, callback: Callable[..., Any]) -> None: ...
    def _get_free_channel_id(self) -> int: ...
    def _inplace_augment_message(
        self, message: Any, exchange: str | None, routing_key: str
    ) -> None: ...
    def _lookup(
        self, exchange: str, routing_key: str, default: str | None = ...
    ) -> list[str] | set[str]: ...
    def _reset_cycle(self) -> None: ...
    @override
    def after_reply_message_received(self, queue: str) -> None: ...
    @override
    def __enter__(self) -> Self: ...
    @override
    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: types.TracebackType | None,
    ) -> None: ...
    @property
    def state(self) -> BrokerState: ...
    @property
    def qos(self) -> _QoSType: ...
    @property
    def cycle(self) -> FairCycle: ...

class Management(BaseManagement):
    transport: Transport  # pyright: ignore[reportIncompatibleVariableOverride]
    channel: _ChannelType | None

    def __init__(self, transport: Transport) -> None: ...
    @override
    def get_bindings(self) -> list[dict[str, Any]]: ...
    def close(self) -> None: ...

class Transport(BaseTransport):
    Channel: type[_ChannelType]
    Cycle: type[FairCycle]
    Management: type[Management]  # pyright: ignore[reportIncompatibleVariableOverride]

    polling_interval: float | None
    channel_max: int
    channels: list[_ChannelType] | None
    cycle: FairCycle | None

    _callbacks: dict[str, Callable[..., Any]] | None

    def __init__(self, client: Connection, **kwargs: Any) -> None: ...
    @override
    def create_channel(self, connection: Connection) -> _ChannelType: ...
    @override
    def close_channel(self, channel: StdChannel) -> None: ...  # pyright: ignore[reportIncompatibleMethodOverride]  # ty: ignore[invalid-method-override]
    @override
    def establish_connection(self) -> Self: ...
    @override
    def close_connection(self, connection: Connection) -> None: ...
    @override
    def drain_events(  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
        self, connection: Connection, timeout: float | None = ...
    ) -> None: ...
    def on_message_ready(
        self, channel: _ChannelType, message: _MessageType, queue: str
    ) -> None: ...
    def _deliver(self, message: Any, queue: str | None) -> None: ...
    def _drain_channel(
        self,
        channel: _ChannelType,
        callback: Callable[..., Any],
        timeout: float | None = ...,
    ) -> None: ...
    def _make_reader(
        self,
        connection: Any,
        timeout: type[BaseException] = ...,
        error: type[BaseException] = ...,
        _unavail: tuple[int, ...] = ...,
    ) -> Callable[..., Any]: ...
    def _reject_inbound_message(self, raw_message: Any) -> None: ...
    @property
    @override
    def default_connection_params(self) -> dict[str, Any]: ...



from typing import Any

__all__ = ("Worker",)

from celery.worker import WorkController
from typing_extensions import override

class Worker(WorkController):
    def emit_banner(self) -> None: ...
    def extra_info(self) -> str: ...
    def install_platform_tweaks(self, worker: WorkController) -> None: ...
    def macOS_proxy_detection_workaround(self) -> None: ...
    @override
    def on_after_init(
        self,
        purge: bool = False,
        no_color: bool | None = None,
        redirect_stdouts: bool | None = None,
        redirect_stdouts_level: str | None = None,
        **kwargs: Any,
    ) -> None: ...
    @override
    def on_before_init(self, quiet: bool = False, **kwargs: Any) -> None: ...
    @override
    def on_consumer_ready(self, consumer: Any) -> None: ...
    @override
    def on_init_blueprint(self) -> None: ...
    @override
    def on_start(self) -> None: ...
    def purge_messages(self) -> int: ...
    def set_process_status(self, info: str) -> None: ...
    def setup_logging(self, colorize: bool | None = None) -> None: ...
    def startup_info(self, artlines: bool = True) -> str: ...
    def tasklist(
        self, include_builtins: bool = True, sep: str = "\n", int_: str = "celery."
    ) -> str: ...



from typing import Any

from kombu.exceptions import LimitExceeded

class Resource:
    LimitExceeded: type[LimitExceeded]
    close_after_fork: bool

    def __init__(
        self,
        limit: int | None = ...,
        preload: int | None = ...,
        close_after_fork: bool | None = ...,
    ) -> None: ...
    def setup(self) -> None: ...
    def acquire(self, block: bool = ..., timeout: float | None = ...) -> Any: ...
    def prepare(self, resource: Any) -> Any: ...
    def close_resource(self, resource: Any) -> None: ...
    def release_resource(self, resource: Any) -> None: ...
    def release(self, resource: Any) -> None: ...
    def collect_resource(self, resource: Any) -> None: ...
    def replace(self, resource: Any) -> None: ...
    def force_close_all(self, close_pool: bool = ...) -> None: ...
    def resize(
        self,
        limit: int,
        force: bool = ...,
        ignore_errors: bool = ...,
        reset: bool = ...,
    ) -> None: ...
    @property
    def limit(self) -> int | None: ...



from logging import Logger
from typing import Any

from kombu.transport.base import Transport as BaseTransport
from kombu.transport.virtual import Channel as VirtualChannel
from kombu.transport.virtual import Message
from kombu.transport.virtual import QoS as VirtualQoS
from typing_extensions import override

logger: Logger
buffer: type[bytes]
OBJECT_ALREADY_EXISTS_STRING: str
VERSION: tuple[int, int, int]

def dependency_is_none(dependency: Any) -> bool: ...

class AuthenticationFailure(Exception): ...

class QoS(VirtualQoS):
    def __init__(self, session: Any, prefetch_count: int = ...) -> None: ...

class Connection:
    Channel: type[Channel]

    def __init__(self, **conn_info: Any) -> None: ...
    def close(self) -> None: ...
    def close_channel(self, channel: Any) -> None: ...
    def get_qpid_connection(self) -> Any: ...

_Message = Message
_QoS = QoS

class Channel(VirtualChannel):
    Message: type[_Message]
    QoS: type[_QoS]  # pyright: ignore[reportIncompatibleVariableOverride]

    def __init__(self, connection: Any, transport: Any) -> None: ...
    @override
    def close(self) -> None: ...
    @override
    def queue_declare(  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
        self,
        queue: str,
        passive: bool = ...,
        durable: bool = ...,
        exclusive: bool = ...,
        auto_delete: bool = ...,
        nowait: bool = ...,
        arguments: dict[str, Any] | None = ...,
    ) -> Any: ...
    @override
    def exchange_declare(  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
        self,
        exchange: str = ...,
        type: str = ...,
        durable: bool = ...,
        **kwargs: Any,
    ) -> None: ...
    @override
    def exchange_delete(  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
        self, exchange_name: str, **kwargs: Any
    ) -> None: ...
    @override
    def queue_bind(  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
        self, queue: str, exchange: str, routing_key: str, **kwargs: Any
    ) -> None: ...
    @override
    def queue_unbind(  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
        self, queue: str, exchange: str, routing_key: str, **kwargs: Any
    ) -> None: ...
    @override
    def basic_qos(  # type: ignore[override]  # pyright: ignore[reportIncompatibleMethodOverride]
        self, prefetch_count: int, *args: Any
    ) -> None: ...

_Connection = Connection

class Transport(BaseTransport):
    Channel: type[Channel]
    Connection: type[_Connection]
    driver_type: str
    driver_name: str
    polling_interval: None
    connection_errors: tuple[type[Exception] | None, ...]  # type: ignore[assignment]  # pyright: ignore[reportIncompatibleVariableOverride]
    channel_errors: tuple[type[Exception] | None, ...]  # type: ignore[assignment]  # pyright: ignore[reportIncompatibleVariableOverride]
    recoverable_connection_errors: tuple[type[BaseException] | None, ...]  # type: ignore[assignment]  # pyright: ignore[reportIncompatibleVariableOverride]
    recoverable_channel_errors: tuple[type[BaseException] | None, ...]  # type: ignore[assignment]  # pyright: ignore[reportIncompatibleVariableOverride]

    def __del__(self) -> None: ...
    @override
    def drain_events(
        self, connection: Any, timeout: int = ..., **kwargs: Any
    ) -> None: ...
    def verify_runtime_environment(self) -> None: ...



from datetime import datetime
from typing import TypedDict

from django.db import models
from django_celery_results.managers import GroupResultManager, TaskResultManager

ALL_STATES: list[str]
TASK_STATE_CHOICES: list[tuple[str, str]]

class TaskResultDict(TypedDict):
    task_id: str
    task_name: str
    task_args: str
    task_kwargs: str
    status: str
    result: str
    date_done: datetime
    traceback: str
    meta: str

class TaskResult(models.Model):
    task_id: models.CharField[str]
    task_name: models.CharField[str | None]
    task_args: models.TextField[str | None]
    task_kwargs: models.TextField[str | None]
    status: models.CharField[str | None]
    content_type: models.CharField[str]
    content_encoding: models.CharField[str]
    result: models.TextField[str | None]
    date_done: models.DateTimeField[datetime]
    traceback: models.TextField[str | None]
    hidden: models.BooleanField[bool]
    meta: models.TextField[str | None]
    objects: TaskResultManager
    def as_dict(self) -> TaskResultDict: ...

class GroupResultDict(TypedDict):
    group_id: str
    result: str | None
    date_done: datetime

class GroupResult(models.Model):
    group_id: models.CharField[str]
    date_created: models.DateTimeField[datetime]
    date_done: models.DateTimeField[datetime]
    content_type: models.CharField[str]
    content_encoding: models.CharField[str]
    result: models.TextField[str | None]
    objects: GroupResultManager
    def as_dict(self) -> GroupResultDict: ...



from logging import Logger, _Level
from multiprocessing.util import Finalize as Finalize
from multiprocessing.util import ForkAwareLocal as ForkAwareLocal
from multiprocessing.util import ForkAwareThreadLock as ForkAwareThreadLock
from multiprocessing.util import get_temp_dir as get_temp_dir
from multiprocessing.util import is_exiting as is_exiting
from multiprocessing.util import register_after_fork as register_after_fork

__all__ = [
    "SUBDEBUG",
    "SUBWARNING",
    "Finalize",
    "ForkAwareLocal",
    "ForkAwareThreadLock",
    "debug",
    "get_logger",
    "get_temp_dir",
    "info",
    "is_exiting",
    "log_to_stderr",
    "register_after_fork",
    "sub_debug",
    "sub_warning",
]

SUBDEBUG: int
SUBWARNING: int

def sub_debug(msg: str, *args: object, **kwargs: object) -> None: ...
def debug(msg: str, *args: object, **kwargs: object) -> None: ...
def info(msg: str, *args: object, **kwargs: object) -> None: ...
def sub_warning(msg: str, *args: object, **kwargs: object) -> None: ...
def warning(msg: str, *args: object, **kwargs: object) -> None: ...
def get_logger() -> Logger: ...
def log_to_stderr(level: _Level | None = ...) -> Logger: ...



# This module is Windows-only and requires msvcrt

from billiard.process import BaseProcess

__all__ = ["Popen"]

class Popen:
    method: str
    returncode: int | None
    pid: int
    sentinel: int | None

    class DupFd:
        def __init__(self, fd: int) -> None: ...
        def detach(self) -> int: ...

    def __init__(self, process_obj: BaseProcess) -> None: ...
    def duplicate_for_child(self, fd: int) -> int: ...
    def poll(self, flag: int = ...) -> int | None: ...
    def wait(self, timeout: float | None = ...) -> int | None: ...
    def terminate(self) -> None: ...



from collections.abc import Callable, Iterator
from contextlib import contextmanager
from importlib.metadata import EntryPoints
from types import ModuleType
from typing import Any

__all__ = (
    "NotAPackage",
    "cwd_in_path",
    "find_module",
    "gen_task_name",
    "import_from_cwd",
    "instantiate",
    "module_file",
    "qualname",
    "reload_from_cwd",
    "symbol_by_name",
)

class NotAPackage(Exception): ...

@contextmanager
def cwd_in_path() -> Iterator[None]: ...
def entry_points(**params: Any) -> EntryPoints: ...
def find_module(
    module: str, path: list[str] | None = None, imp: Any | None = None
) -> ModuleType | None: ...
def gen_task_name(app: Any, name: str, module_name: str) -> str: ...
def import_from_cwd(
    module: str,
    imp: Callable[[str], ModuleType] | None = None,
    package: str | None = None,
) -> ModuleType: ...
def instantiate(name: str, *args: Any, **kwargs: Any) -> Any: ...
def load_extension_class_names(namespace: str) -> list[str]: ...
def load_extension_classes(namespace: str) -> list[type[Any]]: ...
def module_file(module: ModuleType) -> str | None: ...
def qualname(obj: Any) -> str: ...
def reload_from_cwd(
    module: ModuleType, reloader: Callable[[ModuleType], ModuleType] | None = None
) -> ModuleType: ...
def symbol_by_name(
    name: str,
    aliases: dict[str, str] | None = None,
    imp: Callable[[str], ModuleType] | None = None,
    package: str | None = None,
    sep: str = ".",
    default: Any = None,
    **kwargs: Any,
) -> Any: ...



from typing import Any

from celery.backends.base import KeyValueStoreBackend

__all__ = ("CosmosDBSQLBackend",)

class CosmosDBSQLBackend(KeyValueStoreBackend):
    def __init__(
        self,
        url: str | None = None,
        database_name: str | None = None,
        collection_name: str | None = None,
        consistency_level: str | None = None,
        max_retry_attempts: int | None = None,
        max_retry_wait_time: int | None = None,
        *args: Any,
        **kwargs: Any,
    ) -> None: ...



from collections.abc import Callable, Generator, Iterable
from contextlib import contextmanager
from types import TracebackType
from typing import Any, TypeVar

_T = TypeVar("_T")
_ExcT = TypeVar("_ExcT", bound=BaseException)

FILENO_ERRORS: tuple[type[Exception], ...]

class UnsupportedOperation(Exception): ...

def detect_environment() -> str: ...
def entrypoints(namespace: str) -> Iterable[Any]: ...
def fileno(f: Any) -> int: ...
def maybe_fileno(f: Any) -> int | None: ...
@contextmanager
def nested(*managers: Any) -> Generator[tuple[Any, ...], None, None]: ...
def register_after_fork(obj: Any, func: Callable[..., Any]) -> None: ...
def reraise(tp: type[_ExcT], value: _ExcT, tb: TracebackType | None = ...) -> _ExcT: ...
def coro(gen: Callable[..., Generator[_T, Any, Any]]) -> Callable[..., _T]: ...



import socket
from typing import Any

from kombu.entity import Exchange, Queue
from kombu.utils.functional import memoize

#: Exchange for worker direct queues.
WORKER_DIRECT_EXCHANGE = Exchange("C.dq2")

#: Format for worker direct queue names.
WORKER_DIRECT_QUEUE_FORMAT: str = "{hostname}.dq2"

#: Separator for worker node name and hostname.
NODENAME_SEP: str = "@"

NODENAME_DEFAULT: str = "celery"

gethostname = memoize(1, Cache=dict)(socket.gethostname)

__all__ = (
    "anon_nodename",
    "default_nodename",
    "gethostname",
    "host_format",
    "node_format",
    "nodename",
    "nodesplit",
    "worker_direct",
)

def worker_direct(hostname: str | Queue) -> Queue: ...
def nodename(name: str, hostname: str) -> str: ...
def anon_nodename(hostname: str | None = None, prefix: str = "gen") -> str: ...
def nodesplit(name: str) -> tuple[None, str] | list[str]: ...
def default_nodename(hostname: str) -> str: ...
def node_format(s: str, name: str, **extra: dict[str, Any]) -> str: ...
def host_format(
    s: str,
    host: str | None = None,
    name: str | None = None,
    **extra: dict[str, Any],
) -> str: ...



from collections.abc import Callable
from typing import Any

from celery.backends.base import Backend

__all__ = ("AsyncBackendMixin", "BaseResultConsumer", "Drainer", "register_drainer")

def register_drainer(name: str) -> Callable[[type[Drainer]], type[Drainer]]: ...

class Drainer:
    def __init__(self, result_consumer: BaseResultConsumer) -> None: ...
    def start(self) -> None: ...
    def stop(self) -> None: ...
    def drain_events_until(
        self,
        p: Callable[[], bool],
        timeout: float | None = None,
        interval: float = 1,
        on_interval: Callable[[], None] | None = None,
        wait: Callable[[float], None] | None = None,
    ) -> None: ...
    def wait_for(
        self,
        p: Callable[[], bool],
        wait: Callable[[float], None],
        timeout: float | None = None,
    ) -> None: ...

class AsyncBackendMixin:
    @property
    def is_async(self) -> bool: ...
    def wait_for_pending(
        self,
        result: Any,
        callback: Callable[..., Any] | None = None,
        propagate: bool = True,
        **kwargs: Any,
    ) -> Any: ...
    def add_pending_result(
        self, result: Any, weak: bool = False, start_drainer: bool = True
    ) -> Any: ...
    def add_pending_results(self, results: list[Any], weak: bool = False) -> None: ...
    def remove_pending_result(self, result: Any) -> None: ...
    def on_result_fulfilled(self, result: Any) -> None: ...
    def iter_native(self, result: Any, no_ack: bool = True, **kwargs: Any) -> Any: ...

class BaseResultConsumer:
    def __init__(
        self,
        backend: Backend,
        app: Any,
        accept: Any,
        pending_results: Any,
        pending_messages: Any,
    ) -> None: ...
    def start(self, initial_task_id: str, **kwargs: Any) -> None: ...
    def stop(self) -> None: ...
    def drain_events(self, timeout: float | None = None) -> None: ...
    def drain_events_until(
        self,
        p: Callable[[], bool],
        timeout: float | None = None,
        on_interval: Callable[[], None] | None = None,
    ) -> None: ...
    def consume_from(self, task_id: str) -> None: ...
    def cancel_for(self, task_id: str) -> None: ...
    def on_state_change(self, meta: dict[str, Any], message: Any) -> None: ...
    def on_after_fork(self) -> None: ...
    def on_out_of_band_result(self, message: Any) -> None: ...
    def on_wait_for_pending(
        self, result: Any, timeout: float | None = None, **kwargs: Any
    ) -> None: ...



from logging import Logger
from typing import Any

from kombu.transport.virtual import Channel as VirtualChannel
from kombu.transport.virtual import Transport as VirtualTransport
from kombu.transport.virtual.base import BrokerState
from kombu.utils.objects import cached_property
from typing_extensions import override

DEFAULT_PORT: int
E_NAMESERVER: str
E_LOOKUP: str
logger: Logger

class Channel(VirtualChannel):
    def __init__(self, connection: Any, **kwargs: Any) -> None: ...
    @override
    def close(self) -> None: ...
    def queues(self) -> list[str]: ...
    @cached_property
    def shared_queues(self) -> Any: ...

_Channel = Channel

class Transport(VirtualTransport):
    Channel: type[_Channel]  # pyright: ignore[reportIncompatibleVariableOverride]
    driver_type: str
    driver_name: str
    connection_errors: tuple[type[Exception], ...]  # pyright: ignore[reportIncompatibleVariableOverride]
    channel_errors: tuple[type[Exception], ...]  # pyright: ignore[reportIncompatibleVariableOverride]
    global_state: BrokerState
    @cached_property
    def shared_queues(self) -> Any: ...
